# -*- coding: utf-8 -*-
"""Output module for the TLN format.

For documentation on the TLN format see: http://forensicswiki.org/wiki/TLN
"""

from plaso.lib import errors
from plaso.lib import py2to3
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import manager


class TLNBaseOutputModule(interface.LinearOutputModule):
  """Base class for a TLN output module."""
  # Stop pylint from complaining about missing WriteEventBody.
  # pylint: disable=abstract-method

  _FIELD_DELIMITER = u'|'
  _DESCRIPTION_FIELD_DELIMITER = u';'

  _HEADER = u''

  def _FormatDescription(self, event):
    """Formats the description.

    Args:
      event (EventObject): event.

    Returns:
      str: formatted description field.
    """
    date_time_string = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp, timezone=self._output_mediator.timezone)
    timestamp_description = getattr(event, u'timestamp_desc', u'UNKNOWN')

    message, _ = self._output_mediator.GetFormattedMessages(event)
    if message is None:
      data_type = getattr(event, u'data_type', u'UNKNOWN')
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(data_type))

    description = u'{0:s}; {1:s}; {2:s}'.format(
        date_time_string, timestamp_description,
        message.replace(self._DESCRIPTION_FIELD_DELIMITER, u' '))
    return self._SanitizeField(description)

  def _FormatHostname(self, event):
    """Formats the hostname.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted hostname field.
    """
    hostname = self._output_mediator.GetHostname(event)
    return self._SanitizeField(hostname)

  def _FormatSource(self, event):
    """Formats the source.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted source field.
    """
    source_short, _ = self._output_mediator.GetFormattedSources(event)
    if source_short is None:
      data_type = getattr(event, u'data_type', u'UNKNOWN')
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(data_type))

    return self._SanitizeField(source_short)

  def _FormatUsername(self, event):
    """Formats the username.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted username field.
    """
    username = self._output_mediator.GetUsername(event)
    return self._SanitizeField(username)

  def _SanitizeField(self, field):
    """Sanitizes a field for output.

    This method removes the field delimiter from the field string.

    Args:
      field (str): field value.

     Returns:
       str: formatted field value.
    """
    if self._FIELD_DELIMITER and isinstance(field, py2to3.STRING_TYPES):
      return field.replace(self._FIELD_DELIMITER, u' ')
    return field

  def WriteHeader(self):
    """Writes the header to the output."""
    self._WriteLine(self._HEADER)


class TLNOutputModule(TLNBaseOutputModule):
  """Output module for the TLN format.

  TLN defines 5 | separated fields, namely:
  * Time - 32-bit POSIX (or Unix) epoch timestamp.
  * Source - The name of the parser or plugin that produced the event.
  * Host - The source host system.
  * User - The user associated with the data.
  * Description - Message string describing the data.
  """
  NAME = u'tln'
  DESCRIPTION = u'TLN 5 field | delimited output.'

  _HEADER = u'Time|Source|Host|User|Description\n'

  def WriteEventBody(self, event):
    """Writes the body of an event object to the output.

    Args:
      event (EventObject): event.
    """
    if not hasattr(event, u'timestamp'):
      return

    posix_timestamp = timelib.Timestamp.CopyToPosix(event.timestamp)
    source = self._FormatSource(event)
    hostname = self._FormatHostname(event)
    username = self._FormatUsername(event)
    description = self._FormatDescription(event)

    out_write = u'{0:d}|{1:s}|{2:s}|{3:s}|{4!s}\n'.format(
        posix_timestamp, source, hostname, username, description)
    self._WriteLine(out_write)


class L2TTLNOutputModule(TLNBaseOutputModule):
  """Output module for the log2timeline extended variant of the TLN format.

  l2tTLN is an extended variant of TLN introduced log2timeline 0.65.

  l2tTLN extends basic TLN to 7 | separated fields, namely:
  * Time - 32-bit POSIX (or Unix) epoch timestamp.
  * Source - The name of the parser or plugin that produced the event.
  * Host - The source host system.
  * User - The user associated with the data.
  * Description - Message string describing the data.
  * TZ - L2T 0.65 field. Timezone of the event.
  * Notes - L2T 0.65 field. Optional notes field or filename and inode.
  """
  NAME = u'l2ttln'
  DESCRIPTION = u'Extended TLN 7 field | delimited output.'

  _HEADER = u'Time|Source|Host|User|Description|TZ|Notes\n'

  def _FormatNotes(self, event):
    """Formats the notes.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted notes field.
    """
    inode = event.inode
    if inode is None:
      inode = u'-'

    notes = getattr(event, u'notes', u'')
    if not notes:
      display_name = getattr(event, u'display_name', u'')
      notes = u'File: {0:s} inode: {1!s}'.format(display_name, inode)
    return self._SanitizeField(notes)

  def WriteEventBody(self, event):
    """Writes the body of an event object to the output.

    Args:
      event (EventObject): event.
    """
    if not hasattr(event, u'timestamp'):
      return

    posix_timestamp = timelib.Timestamp.CopyToPosix(event.timestamp)
    source = self._FormatSource(event)
    hostname = self._FormatHostname(event)
    username = self._FormatUsername(event)
    description = self._FormatDescription(event)
    notes = self._FormatNotes(event)

    out_write = u'{0:d}|{1:s}|{2:s}|{3:s}|{4:s}|{5!s}|{6!s}\n'.format(
        posix_timestamp, source, hostname, username, description,
        self._output_mediator.timezone, notes)

    self._WriteLine(out_write)


manager.OutputManager.RegisterOutputs([L2TTLNOutputModule, TLNOutputModule])



# -*- coding: utf-8 -*-
"""The output mediator object."""

from plaso.formatters import manager as formatters_manager
from plaso.lib import eventdata

import pytz  # pylint: disable=wrong-import-order


class OutputMediator(object):
  """Class that implements the output mediator.

  Attributes:
    fields_filter (FilterObject): filter object that indicates
                                  which fields to output.
  """

  def __init__(
      self, knowledge_base, formatter_mediator, fields_filter=None,
      preferred_encoding=u'utf-8'):
    """Initializes a output mediator object.

    Args:
      knowledge_base (KnowledgeBase): knowledge base.
      formatter_mediator (FormatterMediator): formatter mediator.
      fields_filter (Optional[FilterObject]): filter object that indicates
          which fields to output.
      preferred_encoding (Optional[str]): preferred encoding to output.
    """
    super(OutputMediator, self).__init__()
    self._formatter_mediator = formatter_mediator
    self._knowledge_base = knowledge_base
    self._preferred_encoding = preferred_encoding
    self._timezone = pytz.UTC

    self.fields_filter = fields_filter

  @property
  def encoding(self):
    """The preferred encoding."""
    return self._preferred_encoding

  @property
  def filter_expression(self):
    """The filter expression if a filter is set, None otherwise."""
    if not self.fields_filter:
      return

    return self.fields_filter.filter_expression

  @property
  def timezone(self):
    """The timezone."""
    return self._timezone

  def GetEventFormatter(self, event):
    """Retrieves the event formatter for a specific event type.

    Args:
      event (EventObject): event.

    Returns:
      The event formatter object (instance of EventFormatter) or None.
    """
    data_type = getattr(event, u'data_type', None)
    if not data_type:
      return

    return formatters_manager.FormattersManager.GetFormatterObject(
        event.data_type)

  def GetFormattedMessages(self, event):
    """Retrieves the formatted messages related to the event.

    Args:
      event (EventObject): event.

    Returns:
      A tuple containing the formatted message string and short message string.
      If no event formatter to match the event can be found the function
      returns a tuple of None, None.
    """
    event_formatter = self.GetEventFormatter(event)
    if not event_formatter:
      return None, None

    return event_formatter.GetMessages(self._formatter_mediator, event)

  def GetFormattedSources(self, event):
    """Retrieves the formatted sources related to the event.

    Args:
      event (EventObject): event.

    Returns:
      A tuple of the short and long source string. If no event formatter
      to match the event can be found the function returns a tuple
      of None, None.
    """
    event_formatter = self.GetEventFormatter(event)
    if not event_formatter:
      return None, None

    return event_formatter.GetSources(event)

  def GetFormatStringAttributeNames(self, event):
    """Retrieves the attribute names in the format string.

    Args:
      event (EventObject): event.

    Returns:
      A list containing the attribute names. If no event formatter to match
      the event can be found the function returns None.
    """
    event_formatter = self.GetEventFormatter(event)
    if not event_formatter:
      return

    return event_formatter.GetFormatStringAttributeNames()

  def GetHostname(self, event, default_hostname=u'-'):
    """Retrieves the hostname related to the event.

    Args:
      event (EventObject): event.
      default_hostname (Optional[str]): default hostname.

    Returns:
      str: hostname.
    """
    hostname = getattr(event, u'hostname', None)
    if hostname:
      return hostname

    session_identifier = event.GetSessionIdentifier()
    if session_identifier is None:
      return default_hostname

    hostname = self._knowledge_base.GetHostname(
        session_identifier=session_identifier)
    return hostname or default_hostname

  # TODO: Fix this function when the MFT parser has been implemented.
  def GetMACBRepresentation(self, event):
    """Retrieves the MACB representation.

    Args:
      event (EventObject): event.

    Returns:
      str: MACB representation.
    """
    data_type = getattr(event, u'data_type', None)
    if not data_type:
      return u'....'

    # The filestat parser is somewhat limited.
    if data_type == u'fs:stat':
      descriptions = event.timestamp_desc.split(u';')

      return_characters = [u'.', u'.', u'.', u'.']
      for description in descriptions:
        if description == u'mtime':
          return_characters[0] = u'M'
        elif description == u'atime':
          return_characters[1] = u'A'
        elif description == u'ctime':
          return_characters[2] = u'C'
        elif description == u'crtime':
          return_characters[3] = u'B'

      return u''.join(return_characters)

    # Access time.
    if event.timestamp_desc in [
        eventdata.EventTimestamp.ACCESS_TIME,
        eventdata.EventTimestamp.ACCOUNT_CREATED,
        eventdata.EventTimestamp.PAGE_VISITED,
        eventdata.EventTimestamp.LAST_VISITED_TIME,
        eventdata.EventTimestamp.START_TIME,
        eventdata.EventTimestamp.LAST_SHUTDOWN,
        eventdata.EventTimestamp.LAST_LOGIN_TIME,
        eventdata.EventTimestamp.LAST_PASSWORD_RESET,
        eventdata.EventTimestamp.LAST_CONNECTED,
        eventdata.EventTimestamp.LAST_RUNTIME,
        eventdata.EventTimestamp.LAST_PRINTED]:
      return u'.A..'

    # Content modification.
    if event.timestamp_desc in [
        eventdata.EventTimestamp.MODIFICATION_TIME,
        eventdata.EventTimestamp.WRITTEN_TIME,
        eventdata.EventTimestamp.DELETED_TIME]:
      return u'M...'

    # Content creation time.
    if event.timestamp_desc in [
        eventdata.EventTimestamp.CREATION_TIME,
        eventdata.EventTimestamp.ADDED_TIME,
        eventdata.EventTimestamp.FILE_DOWNLOADED,
        eventdata.EventTimestamp.FIRST_CONNECTED]:
      return '...B'

    # Metadata modification.
    if event.timestamp_desc in [
        eventdata.EventTimestamp.CHANGE_TIME,
        eventdata.EventTimestamp.ENTRY_MODIFICATION_TIME]:
      return u'..C.'

    return u'....'

  # TODO: remove this function it is incorrect.
  def GetStoredHostname(self):
    """Retrieves the stored hostname.

    Returns:
      str: hostname.
    """
    return self._knowledge_base.GetStoredHostname()

  def GetUsername(self, event, default_username=u'-'):
    """Retrieves the username related to the event.

    Args:
      event (EventObject): event.
      default_username (Optional[str]): default username.

    Returns:
      str: username.
    """
    username = getattr(event, u'username', None)
    if username and username != u'-':
      return username

    session_identifier = event.GetSessionIdentifier()
    if session_identifier is None:
      return default_username

    user_sid = getattr(event, u'user_sid', None)
    username = self._knowledge_base.GetUsernameByIdentifier(
        user_sid, session_identifier=session_identifier)
    return username or default_username

  def SetTimezone(self, timezone):
    """Sets the timezone.

    Args:
      timezone (str): timezone.

    Raises:
      ValueError: if the timezone is not supported.
    """
    if not timezone:
      return

    try:
      self._timezone = pytz.timezone(timezone)
    except pytz.UnknownTimeZoneError:
      raise ValueError(u'Unsupported timezone: {0:s}'.format(timezone))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the event buffer."""

import unittest

from plaso.containers import events
from plaso.output import event_buffer

from tests.cli import test_lib as cli_test_lib
from tests.output import test_lib


class TestEvent(object):
  """Test event object."""

  def __init__(self, timestamp, entry):
    """Initializes an event object."""
    super(TestEvent, self).__init__()
    self.date = u'03/01/2012'
    try:
      self.timestamp = int(timestamp)
    except ValueError:
      self.timestamp = 0
    self.entry = entry


class EventBufferTest(test_lib.OutputModuleTestCase):
  """Tests the event buffer."""

  # pylint: disable=protected-access

  def _CheckBufferLength(self, event_buffer_object, expected_length):
    """Checks the length of the event buffer.

    Args:
      event_buffer_object: the event buffer object (instance of EventBuffer).
      expect_length: the expected event buffer length.
    """
    if not event_buffer_object.check_dedups:
      expected_length = 0

    self.assertEqual(len(event_buffer_object._events_per_key), expected_length)

  def testGetEventIdentifier(self):
    """Tests the _GetEventIdentifier function."""
    output_mediator = self._CreateOutputMediator()
    output_writer = cli_test_lib.TestOutputWriter()
    output_module = test_lib.TestOutputModule(output_mediator)
    output_module.SetOutputWriter(output_writer)
    event_buffer_object = event_buffer.EventBuffer(output_module, False)

    event_a = events.EventObject()
    event_a.timestamp = 123
    event_a.timestamp_desc = u'LAST WRITTEN'
    event_a.data_type = u'mock:nothing'
    event_a.inode = 124
    event_a.filename = u'c:/bull/skrytinmappa/skra.txt'
    event_a.another_attribute = False
    event_a_identifier = event_buffer_object._GetEventIdentifier(event_a)

    event_b = events.EventObject()
    event_b.timestamp = 123
    event_b.timestamp_desc = u'LAST WRITTEN'
    event_b.data_type = u'mock:nothing'
    event_b.inode = 124
    event_b.filename = u'c:/bull/skrytinmappa/skra.txt'
    event_b.another_attribute = False
    event_b_identifier = event_buffer_object._GetEventIdentifier(event_b)

    event_c = events.EventObject()
    event_c.timestamp = 123
    event_c.timestamp_desc = u'LAST UPDATED'
    event_c.data_type = u'mock:nothing'
    event_c.inode = 124
    event_c.filename = u'c:/bull/skrytinmappa/skra.txt'
    event_c.another_attribute = False
    event_c_identifier = event_buffer_object._GetEventIdentifier(event_c)

    event_d = events.EventObject()
    event_d.timestamp = 14523
    event_d.timestamp_desc = u'LAST WRITTEN'
    event_d.data_type = u'mock:nothing'
    event_d.inode = 124
    event_d.filename = u'c:/bull/skrytinmappa/skra.txt'
    event_d.another_attribute = False
    event_d_identifier = event_buffer_object._GetEventIdentifier(event_d)

    event_e = events.EventObject()
    event_e.timestamp = 123
    event_e.timestamp_desc = u'LAST WRITTEN'
    event_e.data_type = u'mock:nothing'
    event_e.inode = 623423
    event_e.filename = u'c:/afrit/öñṅûŗ₅ḱŖūα.txt'
    event_e.another_attribute = False
    event_e_identifier = event_buffer_object._GetEventIdentifier(event_e)

    event_f = events.EventObject()
    event_f.timestamp = 14523
    event_f.timestamp_desc = u'LAST WRITTEN'
    event_f.data_type = u'mock:nothing'
    event_f.inode = 124
    event_f.filename = u'c:/bull/skrytinmappa/skra.txt'
    event_f.another_attribute = False
    event_f.weirdness = u'I am a potato'
    event_f_identifier = event_buffer_object._GetEventIdentifier(event_f)

    self.assertEqual(event_a_identifier, event_b_identifier)
    self.assertNotEqual(event_a_identifier, event_c_identifier)
    self.assertEqual(event_a_identifier, event_e_identifier)
    self.assertNotEqual(event_c_identifier, event_d_identifier)
    self.assertNotEqual(event_d_identifier, event_f_identifier)

  def testFlush(self):
    """Tests the Flush function."""
    output_mediator = self._CreateOutputMediator()
    output_writer = cli_test_lib.TestOutputWriter()
    output_module = test_lib.TestOutputModule(output_mediator)
    output_module.SetOutputWriter(output_writer)
    event_buffer_object = event_buffer.EventBuffer(output_module, False)

    event_buffer_object.Append(TestEvent(123456, u'Now is now'))
    self._CheckBufferLength(event_buffer_object, 1)

    event_buffer_object.Append(TestEvent(123456, u'OMG I AM DIFFERENT'))
    event_buffer_object.Append(TestEvent(123456, u'Now is now'))
    event_buffer_object.Append(TestEvent(123456, u'Now is now'))
    self._CheckBufferLength(event_buffer_object, 2)

    event_buffer_object.Flush()
    self._CheckBufferLength(event_buffer_object, 0)

    event_buffer_object.Append(TestEvent(123456, u'Now is now'))
    event_buffer_object.Append(TestEvent(123456, u'Now is now'))
    event_buffer_object.Append(TestEvent(123456, u'Different again :)'))
    self._CheckBufferLength(event_buffer_object, 2)
    event_buffer_object.Append(TestEvent(123457, u'Now is different'))
    self._CheckBufferLength(event_buffer_object, 1)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the extraction tool object."""

import argparse
import unittest

from plaso.cli import extraction_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class ExtractionToolTest(test_lib.CLIToolTestCase):
  """Tests for the extraction tool object."""

  _EXPECTED_OUTPUT_EXTRACTION_OPTIONS = u'\n'.join([
      (u'usage: extraction_tool_test.py [--hashers HASHER_LIST]'
       u' [--yara_rules PATH]'),
      (u'                               [--parsers PARSER_LIST]'
       u' [--preferred_year YEAR]'),
      u'                               [-p] [--process_archives]',
      u'                               [--skip_compressed_streams]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --hashers HASHER_LIST',
      (u'                        Define a list of hashers to use by the tool. '
       u'This is a'),
      (u'                        comma separated list where each entry is the '
       u'name of a'),
      (u'                        hasher, such as "md5,sha256". "all" '
       u'indicates that all'),
      (u'                        hashers should be enabled. "none" '
       u'disables all'),
      (u'                        hashers. Use "--hashers list" or '
       u'"--info" to list the'),
      u'                        available hashers.',
      u'  --parsers PARSER_LIST',
      (u'                        Define a list of parsers to use by the tool. '
       u'This is a'),
      (u'                        comma separated list where each entry can be '
       u'either a'),
      (u'                        name of a parser or a parser list. Each entry '
       u'can be'),
      (u'                        prepended with an exclamation mark to negate '
       u'the'),
      (u'                        selection (exclude it). The list match is an '
       u'exact'),
      (u'                        match while an individual parser matching is '
       u'a case'),
      (u'                        insensitive substring match, with support for '
       u'glob'),
      (u'                        patterns. Examples would be: "reg" that '
       u'matches the'),
      (u'                        substring "reg" in all parser names or the '
       u'glob'),
      (u'                        pattern "sky[pd]" that would match all '
       u'parsers that'),
      (u'                        have the string "skyp" or "skyd" in its '
       u'name. All'),
      (u'                        matching is case insensitive. Use "--parsers '
       u'list" or'),
      u'                        "--info" to list the available parsers.',
      u'  --preferred_year YEAR, --preferred-year YEAR',
      (u'                        When a format\'s timestamp does not include '
       u'a year,'),
      (u'                        e.g. syslog, use this as the initial year '
       u'instead of'),
      u'                        attempting auto-detection.',
      u'  --process_archives, --process-archives',
      (u'                        Process file entries embedded within archive '
       u'files,'),
      (u'                        such as archive.tar and archive.zip. This '
       u'can make'),
      u'                        processing significantly slower.',
      u'  --skip_compressed_streams, --skip-compressed-streams',
      u'                        Skip processing file content within compressed',
      u'                        streams, such as syslog.gz and syslog.bz2.',
      u'  --yara_rules PATH, --yara-rules PATH',
      (u'                        Path to a file containing Yara rules '
       u'definitions.'),
      (u'  -p, --preprocess      Turn on preprocessing. Preprocessing is '
       u'turned on by'),
      (u'                        default when parsing image files, however if '
       u'a mount'),
      (u'                        point is being parsed then this parameter '
       u'needs to be'),
      u'                        set manually.',
      u''])

  _EXPECTED_PERFORMANCE_OPTIONS = u'\n'.join([
      u'usage: extraction_tool_test.py [--buffer_size BUFFER_SIZE]',
      u'                               [--queue_size QUEUE_SIZE]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      (u'  --buffer_size BUFFER_SIZE, --buffer-size BUFFER_SIZE, '
       u'--bs BUFFER_SIZE'),
      (u'                        The buffer size for the output (defaults to '
       u'196MiB).'),
      u'  --queue_size QUEUE_SIZE, --queue-size QUEUE_SIZE',
      u'                        The maximum number of queued items per worker',
      u'                        (defaults to 125000)',
      u''])

  def testAddExtractionOptions(self):
    """Tests the AddExtractionOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'extraction_tool_test.py', description=u'Test argument parser.',
        add_help=False, formatter_class=test_lib.SortedArgumentsHelpFormatter)

    test_tool = extraction_tool.ExtractionTool()
    test_tool.AddExtractionOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT_EXTRACTION_OPTIONS)

  def testAddPerformanceOptions(self):
    """Tests the AddPerformanceOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'extraction_tool_test.py', description=u'Test argument parser.',
        add_help=False, formatter_class=test_lib.SortedArgumentsHelpFormatter)

    test_tool = extraction_tool.ExtractionTool()
    test_tool.AddPerformanceOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_PERFORMANCE_OPTIONS)

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testParseOptions(self):
    """Tests the ParseOptions function."""
    test_tool = extraction_tool.ExtractionTool()

    options = test_lib.TestOptions()

    # ParseOptions will raise if source is not set.
    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    options.source = self._GetTestFilePath([u'ímynd.dd'])

    test_tool.ParseOptions(options)

    # TODO: improve this test.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the extraction tool object."""

import argparse
import unittest

from plaso.cli import extract_analyze_tool
from plaso.lib import errors

from tests.cli import test_lib


class ExtractionAndAnalysisToolTest(test_lib.CLIToolTestCase):
  """Tests for the extraction and analysis tool object."""

  _STORAGE_FILENAME_TEMPLATE = ur'\d{{8}}T\d{{6}}-{filename}.plaso'

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT_STORAGE_FILE_OPTIONS = u'\n'.join([
      u'usage: extract_analyze_tool_test.py [--storage_file [STORAGE_FILE]]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --storage_file [STORAGE_FILE]',
      u'                        The path of the storage file.',
      u''])

  def testAddStorageFileOptions(self):
    """Tests the AddStorageFileOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'extract_analyze_tool_test.py',
        description=u'Test argument parser.',
        add_help=False, formatter_class=argparse.RawDescriptionHelpFormatter)

    test_tool = extract_analyze_tool.ExtractionAndAnalysisTool()
    test_tool.AddStorageFileOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT_STORAGE_FILE_OPTIONS)

  def testGenerateStorageFileName(self):
    """Tests the _GenerateStorageFileName function."""
    test_tool = extract_analyze_tool.ExtractionAndAnalysisTool()

    source_path = u'/test/storage/path'
    test_tool.SetSourcePath(source_path)
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'path')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    source_path = u'/test/storage/path/'
    test_tool.SetSourcePath(source_path)
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'path')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    source_path = u'/'
    test_tool.SetSourcePath(source_path)
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'ROOT')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    source_path = u'/foo/..'
    test_tool.SetSourcePath(source_path)
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'ROOT')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    source_path = u'foo/../bar'
    test_tool.SetSourcePath(source_path)
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'bar')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    test_tool = extract_analyze_tool.ExtractionAndAnalysisTool()

    options = test_lib.TestOptions()

    # ParseOptions will raise if source is not set.
    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    options.source = self._GetTestFilePath([u'ímynd.dd'])

    test_tool.ParseOptions(options)
    storage_path_regex = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'ímynd.dd')
    self.assertRegexpMatches(test_tool._storage_file_path, storage_path_regex)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the VirusTotal analysis plugin CLI arguments helper."""

import argparse
import unittest

from plaso.analysis import virustotal
from plaso.lib import errors
from plaso.cli.helpers import virustotal_analysis

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class VirusTotalAnalysisArgumentsHelperTest(
    test_lib.AnalysisPluginArgumentsHelperTest):
  """Tests the VirusTotal analysis plugin CLI arguments helper."""

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--virustotal-api-key API_KEY]',
      (u'                     [--virustotal-free-rate-limit] '
       u'[--virustotal-hash HASH]'),
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --virustotal-api-key API_KEY, --virustotal_api_key API_KEY',
      u'                        Specify the API key for use with VirusTotal.',
      u'  --virustotal-free-rate-limit, --virustotal_free_rate_limit',
      (u'                        Limit Virustotal requests to the default '
       u'free API key'),
      (u'                        rate of 4 requests per minute. Set this to '
       u'false if'),
      u'                        you have an key for the private API.',
      u'  --virustotal-hash HASH, --virustotal_hash HASH',
      (u'                        Type of hash to query VirusTotal, the '
       u'default is:'),
      u'                        sha256',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py',
        description=u'Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    virustotal_analysis.VirusTotalAnalysisArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    analysis_plugin = virustotal.VirusTotalAnalysisPlugin()

    with self.assertRaises(errors.BadConfigOption):
      virustotal_analysis.VirusTotalAnalysisArgumentsHelper.ParseOptions(
          options, analysis_plugin)

    options.virustotal_api_key = u'TEST'
    virustotal_analysis.VirusTotalAnalysisArgumentsHelper.ParseOptions(
        options, analysis_plugin)

    with self.assertRaises(errors.BadConfigObject):
      virustotal_analysis.VirusTotalAnalysisArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Android Application Usage history parsers."""

import unittest

from plaso.formatters import android_app_usage  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import android_app_usage

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class AndroidAppUsageParserTest(test_lib.ParserTestCase):
  """Tests for the Android Application Usage History parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'usage-history.xml'])
  def testParse(self):
    """Tests the Parse function."""
    parser_object = android_app_usage.AndroidAppUsageParser()
    storage_writer = self._ParseFile(
        [u'usage-history.xml'], parser_object)

    self.assertEqual(len(storage_writer.events), 28)

    event_object = storage_writer.events[22]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-09 19:28:33.047000')
    self.assertEqual(event_object.timestamp, expected_timestamp)
    self.assertEqual(
        event_object.component,
        u'com.sec.android.widgetapp.ap.hero.accuweather.menu.MenuAdd')

    expected_msg = (
        u'Package: '
        u'com.sec.android.widgetapp.ap.hero.accuweather '
        u'Component: '
        u'com.sec.android.widgetapp.ap.hero.accuweather.menu.MenuAdd')
    expected_msg_short = (
        u'Package: com.sec.android.widgetapp.ap.hero.accuweather '
        u'Component: com.sec.and...')

    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)

    event_object = storage_writer.events[17]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-09-27 19:45:55.675000')
    self.assertEqual(event_object.timestamp, expected_timestamp)
    self.assertEqual(event_object.package, u'com.google.android.gsf.login')

    expected_msg = (
        u'Package: '
        u'com.google.android.gsf.login '
        u'Component: '
        u'com.google.android.gsf.login.NameActivity')
    expected_msg_short = (
        u'Package: com.google.android.gsf.login '
        u'Component: com.google.android.gsf.login...')

    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Class to represent binary data as hexadecimal."""


class Hexdump(object):
  """Class that defines a hexadecimal representation formatter (hexdump)."""

  @classmethod
  def _FormatDataLine(cls, data, data_offset, data_size):
    """Formats binary data in a single line of hexadecimal representation.

    Args:
      data: String containing the binary data.
      data_offset: Offset of the data.
      data_size: Size of the data.

    Returns:
      A Unicode string containing a hexadecimal representation of
      the binary data.

    Raises:
      ValueError: if the data offset is out of bounds.
    """
    if data_offset < 0 or data_offset >= data_size:
      raise ValueError(u'Data offset value out of bounds.')

    if data_size - data_offset > 16:
      data_size = data_offset + 16

    word_values = []
    for byte_offset in range(data_offset, data_size, 2):
      word_value = u'{0:02x}{1:02x}'.format(
          ord(data[byte_offset]), ord(data[byte_offset + 1]))
      word_values.append(word_value)

    byte_values = []
    for byte_offset in range(data_offset, data_size):
      byte_value = ord(data[byte_offset])
      if byte_value > 31 and byte_value < 127:
        byte_value = data[byte_offset]
      else:
        byte_value = u'.'

      byte_values.append(byte_value)

    return u'{0:07x}: {1:s}  {2:s}'.format(
        data_offset, u' '.join(word_values), u''.join(byte_values))

  @classmethod
  def FormatData(cls, data, data_offset=0, maximum_data_size=None):
    """Formats binary data in hexadecimal representation.

    All ASCII characters in the hexadecimal representation (hexdump) are
    translated back to their character representation.

    Args:
      data: String containing the binary data.
      data_offset: Optional offset within the data to start formatting.
      maximum_data_size: Optional maximum size of the data to format.
                         The default is None which represents all of
                         the binary data.

    Returns:
      A Unicode string containing a hexadecimal representation of
      the binary data.
    """
    data_size = len(data)
    if maximum_data_size is not None and maximum_data_size < data_size:
      data_size = maximum_data_size

    output_strings = []
    for line_offset in range(data_offset, data_size, 16):
      hexdump_line = cls._FormatDataLine(data, line_offset, data_size)
      output_strings.append(hexdump_line)

    return u'\n'.join(output_strings)



# -*- coding: utf-8 -*-
"""This file is the template for Plist events."""

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import eventdata
from plaso.lib import timelib


class PlistEvent(time_events.TimestampEvent):
  """Convenience class for a plist events."""

  DATA_TYPE = u'plist:key'

  def __init__(self, root, key, datetime_time, desc=None, host=None, user=None):
    """Template for creating a Plist EventObject for returning data to Plaso.

    All events extracted from files get passed around Plaso internally as an
    EventObject. PlistEvent is an EventObject with attributes specifically
    relevant to data extracted from a Plist file. The attribute DATA_TYPE
    'plist:key' allows the formatter used during output to identify
    the appropriate formatter for converting these attributes to output.

    Args:
      root (str): path from the root to this key.
      key (str): name of key.
      datetime_time (datetime.datetime): datetime.
      desc (Optional[str]): description.
      host (Optional[str]): name of host.
      user (Optional[str]): name of user.
    """
    timestamp = timelib.Timestamp.FromPythonDatetime(datetime_time)
    super(PlistEvent, self).__init__(
        timestamp, eventdata.EventTimestamp.WRITTEN_TIME)

    self.root = root
    self.key = key
    if desc:
      self.desc = desc
    if host:
      self.hostname = host
    if user:
      self.username = user


# TODO: remove the need for this class.
class PlistTimeEventData(events.EventData):
  """Plist event data.

  Attributes:
    desc (str): description.
    host (str): hostname.
    key (str): name of plist key.
    root (str): path from the root to this plist key.
    user (str): unique username.
  """

  DATA_TYPE = u'plist:key'

  def __init__(self):
    """Initializes event data."""
    super(PlistTimeEventData, self).__init__(data_type=self.DATA_TYPE)
    self.desc = None
    self.hostname = None
    self.key = None
    self.root = None
    self.username = None



# -*- coding: utf-8 -*-
"""The CLI tools classes."""

import abc
import datetime
import locale
import logging
import os
import sys

try:
  import win32api
  import win32console
except ImportError:
  win32console = None

import plaso

from plaso.cli import views
from plaso.engine import engine
from plaso.lib import errors
from plaso.lib import py2to3

import pytz  # pylint: disable=wrong-import-order


class CLITool(object):
  """Class that implements a CLI tool.

  Attributes:
    list_profilers (bool): True if the profilers should be listed.
    list_timezones (bool): True if the time zones should be listed.
    preferred_encoding (str): preferred encoding of single-byte or multi-byte
        character strings, sometimes referred to as extended ASCII.
  """
  # The maximum number of characters of a line written to the output writer.
  _LINE_LENGTH = 80

  # The fall back preferred encoding.
  _PREFERRED_ENCODING = u'utf-8'

  _PROFILERS_INFORMATION = {
      u'parsers': u'Profile CPU time per parser',
      u'processing': u'Profile CPU time of processing phases',
      u'serializers': u'Profile CPU time of serialization'}

  if engine.BaseEngine.SupportsGuppyMemoryProfiling():
    _PROFILERS_INFORMATION[u'guppy'] = (
        u'Profile memory usage per process using guppy')

  _DEFAULT_PROFILING_SAMPLE_RATE = 1000

  NAME = u''

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(CLITool, self).__init__()

    preferred_encoding = locale.getpreferredencoding()
    if not preferred_encoding:
      preferred_encoding = self._PREFERRED_ENCODING

    if not input_reader:
      input_reader = StdinInputReader(encoding=preferred_encoding)
    if not output_writer:
      output_writer = StdoutOutputWriter(encoding=preferred_encoding)

    self._data_location = None
    self._debug_mode = False
    self._encode_errors = u'strict'
    self._input_reader = input_reader
    self._log_file = None
    self._output_writer = output_writer
    self._preferred_time_zone = None
    self._profilers = set()
    self._profiling_directory = None
    self._profiling_sample_rate = self._DEFAULT_PROFILING_SAMPLE_RATE
    self._quiet_mode = False
    self._views_format_type = views.ViewsFactory.FORMAT_TYPE_CLI

    self.list_profilers = False
    self.list_timezones = False
    self.preferred_encoding = preferred_encoding

  def _ClearScreen(self):
    """Clears the terminal/console screen."""
    if not win32console:
      # ANSI escape sequence to clear screen.
      self._output_writer.Write(b'\033[2J')
      # ANSI escape sequence to move cursor to top left.
      self._output_writer.Write(b'\033[H')

    else:
      # Windows cmd.exe does not support ANSI escape codes, thus instead we
      # fill the console screen buffer with spaces.
      top_left_coordinate = win32console.PyCOORDType(0, 0)
      screen_buffer = win32console.GetStdHandle(win32api.STD_OUTPUT_HANDLE)
      screen_buffer_information = screen_buffer.GetConsoleScreenBufferInfo()

      screen_buffer_attributes = screen_buffer_information[u'Attributes']
      screen_buffer_size = screen_buffer_information[u'Size']
      console_size = screen_buffer_size.X * screen_buffer_size.Y

      screen_buffer.FillConsoleOutputCharacter(
          u' ', console_size, top_left_coordinate)
      screen_buffer.FillConsoleOutputAttribute(
          screen_buffer_attributes, console_size, top_left_coordinate)
      screen_buffer.SetConsoleCursorPosition(top_left_coordinate)

  def _ConfigureLogging(
      self, filename=None, format_string=None, log_level=None):
    """Configure the logger.

    If a filename is specified and the corresponding log file already exists,
    the file is truncated.

    Args:
      filename (Optional[str]): path to a filename to append logs to, where
          None means logs will not be redirected to a file.
      format_string (Optional[str]): format string for the logs, where None
           configures the logger to use a default format string.
      log_level (Optional[int]): integer representing the log level, for
          example logging.DEBUG, where None represents logging.INFO.
    """
    # Remove all possible log handlers.
    for handler in logging.root.handlers:
      logging.root.removeHandler(handler)

    if log_level is None:
      log_level = logging.INFO

    if not format_string:
      format_string = u'[%(levelname)s] %(message)s'

    if filename:
      logging.basicConfig(
          level=log_level, format=format_string, filename=filename,
          filemode=u'w')
    else:
      logging.basicConfig(level=log_level, format=format_string)

  def _EncodeString(self, string):
    """Encodes a string in the preferred encoding.

    Returns:
      bytes: encoded string.
    """
    try:
      # Note that encode() will first convert string into a Unicode string
      # if necessary.
      encoded_string = string.encode(
          self.preferred_encoding, errors=self._encode_errors)
    except UnicodeEncodeError:
      if self._encode_errors == u'strict':
        logging.error(
            u'Unable to properly write output due to encoding error. '
            u'Switching to error tolerant encoding which can result in '
            u'non Basic Latin (C0) characters to be replaced with "?" or '
            u'"\\ufffd".')
        self._encode_errors = u'replace'

      encoded_string = string.encode(
          self.preferred_encoding, errors=self._encode_errors)

    return encoded_string

  def _ParseDataLocationOption(self, options):
    """Parses the data location option.

    Args:
      options (argparse.Namespace): command line arguments.
    """
    data_location = self.ParseStringOption(options, u'data_location')
    if not data_location:
      # Determine if we are running from the source directory.
      # This should get us the path to the "plaso/cli" directory.
      data_location = os.path.dirname(__file__)

      # In order to get to the main path of the egg file we need to traverse
      # two directories up.
      data_location = os.path.dirname(data_location)
      data_location = os.path.dirname(data_location)

      # There are multiple options to run a tool e.g. running from source or
      # from an egg file.
      data_location_egg = os.path.join(data_location, u'share', u'plaso')
      data_location_source = os.path.join(data_location, u'data')
      data_location_system = os.path.join(sys.prefix, u'share', u'plaso')
      data_location_system_local = os.path.join(
          sys.prefix, u'local', u'share', u'plaso')

      if os.path.exists(data_location_egg):
        data_location = data_location_egg
      elif os.path.exists(data_location_source):
        data_location = data_location_source
      elif os.path.exists(data_location_system):
        data_location = data_location_system
      elif os.path.exists(data_location_system_local):
        data_location = data_location_system_local
      else:
        data_location = None

    if not data_location:
      self._output_writer.Write(
          u'WARNING: unable to determine location of data files.\n')

    self._data_location = data_location

  def _ParseInformationalOptions(self, options):
    """Parses the informational options.

    Args:
      options (argparse.Namespace): command line arguments.
    """
    self._debug_mode = getattr(options, u'debug', False)
    self._quiet_mode = getattr(options, u'quiet', False)

    if self._debug_mode and self._quiet_mode:
      logging.warning(
          u'Cannot use debug and quiet mode at the same time, defaulting to '
          u'debug output.')

  def _ParseProfilingOptions(self, options):
    """Parses the profiling options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    profilers_string = self.ParseStringOption(options, u'profilers')
    if isinstance(profilers_string, py2to3.STRING_TYPES):
      if profilers_string.lower() == u'list':
        self.list_profilers = True

      elif profilers_string:
        profilers = set(profilers_string.split(u','))

        supported_profilers = set(self._PROFILERS_INFORMATION.keys())
        unsupported_profilers = profilers.difference(supported_profilers)
        if unsupported_profilers:
          unsupported_profilers = u', '.join(unsupported_profilers)
          raise errors.BadConfigOption(
              u'Unsupported profilers: {0:s}'.format(unsupported_profilers))

        self._profilers = profilers

    self._profiling_directory = getattr(options, u'profiling_directory', None)
    if (self._profiling_directory and
        not os.path.isdir(self._profiling_directory)):
      raise errors.BadConfigOption(
          u'No such profiling directory: {0:s}'.format(
              self._profiling_directory))

    profiling_sample_rate = getattr(options, u'profiling_sample_rate', None)
    if profiling_sample_rate:
      try:
        self._profiling_sample_rate = int(profiling_sample_rate, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid profile sample rate: {0:s}.'.format(
                profiling_sample_rate))

  def _ParseTimezoneOption(self, options):
    """Parses the timezone options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    time_zone_string = self.ParseStringOption(options, u'timezone')
    if isinstance(time_zone_string, py2to3.STRING_TYPES):
      if time_zone_string.lower() == u'list':
        self.list_timezones = True

      elif time_zone_string:
        try:
          pytz.timezone(time_zone_string)
        except pytz.UnknownTimeZoneError:
          raise errors.BadConfigOption(
              u'Unknown time zone: {0:s}'.format(time_zone_string))

        self._preferred_time_zone = time_zone_string

  def AddBasicOptions(self, argument_group):
    """Adds the basic options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    version_string = u'plaso - {0:s} version {1:s}'.format(
        self.NAME, plaso.__version__)

    # We want a custom help message and not the default argparse one.
    argument_group.add_argument(
        u'-h', u'--help', action=u'help',
        help=u'show this help message and exit.')

    argument_group.add_argument(
        u'-V', u'--version', dest=u'version', action=u'version',
        version=version_string, help=u'show the version information.')

  def AddDataLocationOption(self, argument_group):
    """Adds the data location option to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--data', action=u'store', dest=u'data_location', type=str,
        metavar=u'PATH', default=None, help=u'the location of the data files.')

  def AddInformationalOptions(self, argument_group):
    """Adds the informational options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        '-d', '--debug', dest='debug', action='store_true', default=False,
        help=u'enable debug output.')

    argument_group.add_argument(
        '-q', '--quiet', dest='quiet', action='store_true', default=False,
        help=u'disable informational output.')

  def AddLogFileOptions(self, argument_group):
    """Adds the log file option to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--logfile', u'--log_file', u'--log-file', action=u'store',
        metavar=u'FILENAME', dest=u'log_file', type=str, default=u'', help=(
            u'If defined all log messages will be redirected to this file '
            u'instead the default STDERR.'))

  def AddProfilingOptions(self, argument_group):
    """Adds the profiling options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--profilers', dest=u'profilers', type=str, action=u'store',
        default=u'', metavar=u'PROFILERS_LIST', help=(
            u'Define a list of profilers to use by the tool. This is a comma '
            u'separated list where each entry is the name of a profiler. '
            u'Use "--profilers list" to list the available profilers.'))

    argument_group.add_argument(
        u'--profiling_directory', u'--profiling-directory',
        dest=u'profiling_directory', type=str, action=u'store',
        metavar=u'DIRECTORY', help=(
            u'Path to the directory that should be used to store the profiling '
            u'sample files. By default the sample files are stored in the '
            u'current working directory.'))

    argument_group.add_argument(
        u'--profiling_sample_rate', u'--profiling-sample-rate',
        dest=u'profiling_sample_rate', action=u'store', metavar=u'SAMPLE_RATE',
        default=0, help=(
            u'The profiling sample rate (defaults to a sample every {0:d} '
            u'files).').format(self._DEFAULT_PROFILING_SAMPLE_RATE))

  def AddTimeZoneOption(self, argument_group):
    """Adds the time zone option to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'-z', u'--zone', u'--timezone', dest=u'timezone', action=u'store',
        type=str, default=u'UTC', help=(
            u'explicitly define the timezone. Typically the timezone is '
            u'determined automatically where possible. Use "-z list" to '
            u'see a list of available timezones.'))

  def GetCommandLineArguments(self):
    """Retrieves the command line arguments.

    Returns:
      str: command line arguments.
    """
    command_line_arguments = sys.argv
    if not command_line_arguments:
      return u''

    if isinstance(command_line_arguments[0], py2to3.BYTES_TYPE):
      encoding = sys.stdin.encoding

      # Note that sys.stdin.encoding can be None.
      if not encoding:
        encoding = self.preferred_encoding

      try:
        command_line_arguments = [
            argument.decode(encoding) for argument in command_line_arguments]

      except UnicodeDecodeError:
        logging.error(
            u'Unable to properly read command line input due to encoding '
            u'error. Replacing non Basic Latin (C0) characters with "?" or '
            u'"\\ufffd".')

        command_line_arguments = [
            argument.decode(encoding, errors=u'replace')
            for argument in command_line_arguments]

    return u' '.join(command_line_arguments)

  def ListProfilers(self):
    """Lists information about the available profilers."""
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, column_names=[u'Name', u'Description'],
        title=u'Profilers')

    for name, description in sorted(self._PROFILERS_INFORMATION.items()):
      table_view.AddRow([name, description])
    table_view.Write(self._output_writer)

  def ListTimeZones(self):
    """Lists the timezones."""
    max_length = 0
    for timezone_name in pytz.all_timezones:
      if len(timezone_name) > max_length:
        max_length = len(timezone_name)

    utc_date_time = datetime.datetime.utcnow()

    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, column_names=[u'Timezone', u'UTC Offset'],
        title=u'Zones')
    for timezone_name in pytz.all_timezones:
      local_timezone = pytz.timezone(timezone_name)

      local_date_string = u'{0!s}'.format(
          local_timezone.localize(utc_date_time))
      if u'+' in local_date_string:
        _, _, diff = local_date_string.rpartition(u'+')
        diff_string = u'+{0:s}'.format(diff)
      else:
        _, _, diff = local_date_string.rpartition(u'-')
        diff_string = u'-{0:s}'.format(diff)

      table_view.AddRow([timezone_name, diff_string])

    table_view.Write(self._output_writer)

  def ParseOptions(self, options):
    """Parses tool specific options.

    Args:
      options (argparse.Namespace): command line arguments.
    """
    self._ParseInformationalOptions(options)

  def ParseLogFileOptions(self, options):
    """Parses the log file options.

    Args:
      options (argparse.Namespace): command line arguments.
    """
    self._log_file = self.ParseStringOption(options, u'log_file')

  def ParseStringOption(self, options, argument_name, default_value=None):
    """Parses a string command line argument.

    Args:
      options (argparse.Namespace): command line arguments.
      argument_name (str): name of the command line argument.
      default_value (Optional[object]): default value of the command line
          argument.

    Returns:
      object: command line argument value. If the command line argument is
          not set the default value will be returned.

    Raises:
      BadConfigOption: if the command line argument value cannot be converted
                       to a Unicode string.
    """
    argument_value = getattr(options, argument_name, None)
    if not argument_value:
      return default_value

    if isinstance(argument_value, py2to3.BYTES_TYPE):
      encoding = sys.stdin.encoding

      # Note that sys.stdin.encoding can be None.
      if not encoding:
        encoding = self.preferred_encoding

      try:
        argument_value = argument_value.decode(encoding)
      except UnicodeDecodeError as exception:
        raise errors.BadConfigOption((
            u'Unable to convert option: {0:s} to Unicode with error: '
            u'{1:s}.').format(argument_name, exception))

    elif not isinstance(argument_value, py2to3.UNICODE_TYPE):
      raise errors.BadConfigOption(
          u'Unsupported option: {0:s} string type required.'.format(
              argument_name))

    return argument_value

  def PrintSeparatorLine(self):
    """Prints a separator line."""
    self._output_writer.Write(u'-' * self._LINE_LENGTH)
    self._output_writer.Write(u'\n')


class CLIInputReader(object):
  """Class that implements the CLI input reader interface."""

  def __init__(self, encoding=u'utf-8'):
    """Initializes the input reader object.

    Args:
      encoding (Optional[str]): input encoding.
    """
    super(CLIInputReader, self).__init__()
    self._encoding = encoding

  @abc.abstractmethod
  def Read(self):
    """Reads a string from the input.

    Returns:
      str: input.
    """


class CLIOutputWriter(object):
  """Class that implements the CLI output writer interface."""

  def __init__(self, encoding=u'utf-8'):
    """Initializes the output writer object.

    Args:
      encoding (Optional[str]): output encoding.
    """
    super(CLIOutputWriter, self).__init__()
    self._encoding = encoding

  @abc.abstractmethod
  def Write(self, string):
    """Writes a string to the output.

    Args:
      string (str): output.
    """


class FileObjectInputReader(CLIInputReader):
  """Class that implements a file-like object input reader.

  This input reader relies on the file-like object having a readline method.
  """

  def __init__(self, file_object, encoding=u'utf-8'):
    """Initializes the input reader object.

    Args:
      file_object (file): file-like object to read from.
      encoding (Optional[str]): input encoding.
    """
    super(FileObjectInputReader, self).__init__(encoding=encoding)
    self._errors = u'strict'
    self._file_object = file_object

  def Read(self):
    """Reads a string from the input.

    Returns:
      str: input.
    """
    encoded_string = self._file_object.readline()

    try:
      string = encoded_string.decode(self._encoding, errors=self._errors)
    except UnicodeDecodeError:
      if self._errors == u'strict':
        logging.error(
            u'Unable to properly read input due to encoding error. '
            u'Switching to error tolerant encoding which can result in '
            u'non Basic Latin (C0) characters to be replaced with "?" or '
            u'"\\ufffd".')
        self._errors = u'replace'

      string = encoded_string.decode(self._encoding, errors=self._errors)

    return string


class StdinInputReader(FileObjectInputReader):
  """Class that implements a stdin input reader."""

  def __init__(self, encoding=u'utf-8'):
    """Initializes the input reader object.

    Args:
      encoding (Optional[str]): input encoding.
    """
    super(StdinInputReader, self).__init__(sys.stdin, encoding=encoding)


class FileObjectOutputWriter(CLIOutputWriter):
  """Class that implements a file-like object output writer.

  This output writer relies on the file-like object having a write method.
  """

  def __init__(self, file_object, encoding=u'utf-8'):
    """Initializes the output writer object.

    Args:
      file_object (file): file-like object to read from.
      encoding (Optional[str]): output encoding.
    """
    super(FileObjectOutputWriter, self).__init__(encoding=encoding)
    self._errors = u'strict'
    self._file_object = file_object

  def Write(self, string):
    """Writes a string to the output.

    Args:
      string (str): output.
    """
    try:
      # Note that encode() will first convert string into a Unicode string
      # if necessary.
      encoded_string = string.encode(self._encoding, errors=self._errors)
    except UnicodeEncodeError:
      if self._errors == u'strict':
        logging.error(
            u'Unable to properly write output due to encoding error. '
            u'Switching to error tolerant encoding which can result in '
            u'non Basic Latin (C0) characters to be replaced with "?" or '
            u'"\\ufffd".')
        self._errors = u'replace'

      encoded_string = string.encode(self._encoding, errors=self._errors)

    self._file_object.write(encoded_string)


class StdoutOutputWriter(FileObjectOutputWriter):
  """Class that implements a stdout output writer."""

  def __init__(self, encoding=u'utf-8'):
    """Initializes the output writer object.

    Args:
      encoding (Optional[str]): output encoding.
    """
    super(StdoutOutputWriter, self).__init__(sys.stdout, encoding=encoding)



# -*- coding: utf-8 -*-
"""The common front-end functionality."""


class Frontend(object):
  """Class that implements a front-end."""


class TimeSlice(object):
  """Class that defines a time slice.

  The time slice is used to provide a context of events around an event
  of interest.

  Attributes:
    duration: duration of the time slize in minutes.
    event_timestamp: event timestamp of the time slice or None.
  """

  _MICRO_SECONDS_PER_MINUTE = 60 * 1000000

  def __init__(self, event_timestamp, duration=5):
    """Initializes the time slice.

    Args:
      event_timestamp: event timestamp of the time slice or None.
      duration: optional duration of the time slize in minutes.
                The default is 5, which represent 2.5 minutes before
                and 2.5 minutes after the event timestamp.
    """
    super(TimeSlice, self).__init__()
    self.duration = duration
    self.event_timestamp = event_timestamp

  @property
  def end_timestamp(self):
    """The slice end timestamp or None."""
    if not self.event_timestamp:
      return
    return self.event_timestamp + (
        self.duration * self._MICRO_SECONDS_PER_MINUTE)

  @property
  def start_timestamp(self):
    """The slice start timestamp or None."""
    if not self.event_timestamp:
      return
    return self.event_timestamp - (
        self.duration * self._MICRO_SECONDS_PER_MINUTE)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the log2timeline front-end."""

import unittest

from plaso.frontend import log2timeline

from tests import test_lib as shared_test_lib


class Log2TimelineFrontendTest(shared_test_lib.BaseTestCase):
  """Tests for the log2timeline front-end."""

  # pylint: disable=protected-access

  def testGetFiltersInformation(self):
    """Tests the _GetFiltersInformation function."""
    test_front_end = log2timeline.Log2TimelineFrontend()
    filters_info = test_front_end._GetFiltersInformation()

    self.assertIsNotNone(filters_info)

    available_filter_names = [name for name, _ in filters_info]
    self.assertIn(u'DynamicFilter', available_filter_names)
    self.assertIn(u'EventObjectFilter', available_filter_names)

  def testGetOutputModulesInformation(self):
    """Tests the _GetOutputModulesInformation function."""
    test_front_end = log2timeline.Log2TimelineFrontend()
    modules_info = test_front_end._GetOutputModulesInformation()

    self.assertIsNotNone(modules_info)

    available_module_names = [name for name, _ in modules_info]
    self.assertIn(u'dynamic', available_module_names)
    self.assertIn(u'json', available_module_names)

  def testGetDisabledOutputClasses(self):
    """Tests the GetDisabledOutputClasses function."""
    test_front_end = log2timeline.Log2TimelineFrontend()
    output_classes = list(test_front_end.GetDisabledOutputClasses())

    self.assertIsNotNone(output_classes)

  def testGetOutputClasses(self):
    """Tests the GetOutputClasses function."""
    test_front_end = log2timeline.Log2TimelineFrontend()
    output_classes = list(test_front_end.GetOutputClasses())

    self.assertIsNotNone(output_classes)

    available_output_names = [name for name, _ in output_classes]
    self.assertIn(u'dynamic', available_output_names)
    self.assertIn(u'json', available_output_names)

  def testGetPluginData(self):
    """Tests the GetPluginData function."""
    test_front_end = log2timeline.Log2TimelineFrontend()
    plugin_info = test_front_end.GetPluginData()

    self.assertIn(u'Hashers', plugin_info)

    available_hasher_names = [name for name, _ in plugin_info[u'Hashers']]
    self.assertIn(u'sha256', available_hasher_names)
    self.assertIn(u'sha1', available_hasher_names)

    self.assertIn(u'Parsers', plugin_info)
    self.assertIsNotNone(plugin_info[u'Parsers'])

    self.assertIn(u'Parser Plugins', plugin_info)
    self.assertIsNotNone(plugin_info[u'Parser Plugins'])


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Parser test for Mac Cups IPP Log files."""

import unittest

from plaso.formatters import cups_ipp  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import cups_ipp

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class CupsIppParserTest(test_lib.ParserTestCase):
  """The unit test for Mac Cups IPP parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'mac_cups_ipp'])
  def testParse(self):
    """Tests the Parse function."""
    # TODO: only tested against Mac OS X Cups IPP (Version 2.0)
    parser_object = cups_ipp.CupsIppParser()
    storage_writer = self._ParseFile(
        [u'mac_cups_ipp'], parser_object)

    self.assertEqual(len(storage_writer.events), 3)

    events = self._GetSortedEvents(storage_writer.events)

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-03 18:07:21')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc,
        eventdata.EventTimestamp.CREATION_TIME)

    self.assertEqual(event.application, u'LibreOffice')
    self.assertEqual(event.job_name, u'Assignament 1')
    self.assertEqual(event.computer_name, u'localhost')
    self.assertEqual(event.copies, 1)
    self.assertEqual(event.doc_type, u'application/pdf')
    expected_job = u'urn:uuid:d51116d9-143c-3863-62aa-6ef0202de49a'
    self.assertEqual(event.job_id, expected_job)
    self.assertEqual(event.owner, u'Joaquin Moreno Garijo')
    self.assertEqual(event.user, u'moxilo')
    self.assertEqual(event.printer_id, u'RHULBW')
    expected_uri = u'ipp://localhost:631/printers/RHULBW'
    self.assertEqual(event.uri, expected_uri)
    expected_msg = (
        u'User: moxilo '
        u'Owner: Joaquin Moreno Garijo '
        u'Job Name: Assignament 1 '
        u'Application: LibreOffice '
        u'Printer: RHULBW')
    expected_msg_short = (
        u'Job Name: Assignament 1')
    self._TestGetMessageStrings(event, expected_msg, expected_msg_short)

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-03 18:07:21')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc,
        eventdata.EventTimestamp.START_TIME)

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-03 18:07:32')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc,
        eventdata.EventTimestamp.END_TIME)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""File system stat object parser."""

from dfvfs.lib import definitions as dfvfs_definitions

from plaso.containers import time_events
from plaso.lib import timelib
from plaso.parsers import interface
from plaso.parsers import manager


class FileStatEvent(time_events.TimestampEvent):
  """File system stat event.

  Attributes:
    file_entry_type (int): dfVFS file entry type.
    file_size (int): file size in bytes.
    file_system_type (str): file system type.
    is_allocated (bool): True if the file is allocated.
    offset (int): the offset of the stat data in bytes.
  """

  DATA_TYPE = u'fs:stat'

  def __init__(
      self, timestamp, timestamp_description, is_allocated, file_size,
      file_entry_type, file_system_type):
    """Initializes the event object.

    Args:
      timestamp (int): timestamp, which contains the number of microseconds
          since Jan 1, 1970 00:00:00 UTC
      timestamp_description (str): description of the timestamp.
      is_allocated (bool): True if the file entry is allocated.
      file_size (int): file size in bytes.
      file_entry_type (int): dfVFS file entry type.
      file_system_type (str): file system type.
    """
    super(FileStatEvent, self).__init__(timestamp, timestamp_description)
    self.file_entry_type = file_entry_type
    self.file_size = file_size
    self.file_system_type = file_system_type
    self.is_allocated = is_allocated
    self.offset = 0


class FileStatParser(interface.FileEntryParser):
  """Parses file system stat object."""

  NAME = u'filestat'
  DESCRIPTION = u'Parser for file system stat information.'

  _TIME_ATTRIBUTES = frozenset([
      u'atime', u'bkup_time', u'ctime', u'crtime', u'dtime', u'mtime'])

  def _GetFileSystemTypeFromFileEntry(self, file_entry):
    """Return a filesystem type string from a file entry object.

    Args:
      file_entry (dfvfs.FileEntry): a file entry.

    Returns:
      str: file system type.
    """
    if file_entry.type_indicator != dfvfs_definitions.TYPE_INDICATOR_TSK:
      return file_entry.type_indicator

    # TODO: Implement fs_type in dfVFS and remove this implementation
    # once that is in place.
    file_system = file_entry.GetFileSystem()
    fs_info = file_system.GetFsInfo()
    if fs_info.info:
      type_string = u'{0:s}'.format(fs_info.info.ftype)
      if type_string.startswith(u'TSK_FS_TYPE'):
        return type_string[12:]

  def ParseFileEntry(self, parser_mediator, file_entry, **kwargs):
    """Parses a file entry.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_entry (dfvfs.FileEntry): a file entry.
    """
    stat_object = file_entry.GetStat()
    if not stat_object:
      return

    file_system_type = self._GetFileSystemTypeFromFileEntry(file_entry)

    is_allocated = getattr(stat_object, u'allocated', True)
    file_size = getattr(stat_object, u'size', None),

    for time_attribute in self._TIME_ATTRIBUTES:
      posix_time = getattr(stat_object, time_attribute, None)
      if posix_time is None:
        continue

      nano_time_attribute = u'{0:s}_nano'.format(time_attribute)
      nano_time_attribute = getattr(stat_object, nano_time_attribute, None)

      timestamp = timelib.Timestamp.FromPosixTime(posix_time)
      if nano_time_attribute is not None:
        # Note that the _nano values are in intervals of 100th nano seconds.
        micro_time_attribute, _ = divmod(nano_time_attribute, 10)
        timestamp += micro_time_attribute

      # TSK will return 0 if the timestamp is not set.
      if (file_entry.type_indicator == dfvfs_definitions.TYPE_INDICATOR_TSK and
          not timestamp):
        continue

      event_object = FileStatEvent(
          timestamp, time_attribute, is_allocated, file_size, stat_object.type,
          file_system_type)
      parser_mediator.ProduceEvent(event_object)


manager.ParsersManager.RegisterParser(FileStatParser)



# -*- coding: utf-8 -*-
"""This file contains the event filters manager class."""

import logging

from plaso.lib import errors


class FiltersManager(object):
  """Class that implements the filters manager."""

  _filter_classes = {}

  @classmethod
  def DeregisterFilter(cls, filter_class):
    """Deregisters a filter class.

    The filter classes are identified based on their lower case filter name.

    Args:
      filter_class: the class object of the filter.

    Raises:
      KeyError: if filter class is not set for the corresponding filter name.
    """
    filter_name = filter_class.__name__
    if filter_name not in cls._filter_classes:
      raise KeyError(
          u'Filter class not set for filter name: {0:s}.'.format(
              filter_class.__name__))

    del cls._filter_classes[filter_name]

  @classmethod
  def GetFilterObject(cls, filter_string):
    """Creates instances of specific filters.

    Args:
      filter_string: A filter string for any of the available filters.

    Returns:
     The first filter found matching the filter string or None if no
     corresponding filter is available.
    """
    if not filter_string:
      return

    # TODO: refactor not to instantiate all filter classes.
    for filter_object in cls.GetFilterObjects():
      try:
        filter_object.CompileFilter(filter_string)
        return filter_object

      except errors.WrongPlugin:
        logging.debug(u'Filter string [{0:s}] is not a filter: {1:s}'.format(
            filter_string, filter_object.filter_name))

  @classmethod
  def GetFilterObjects(cls):
    """Creates instances of the available filters.

    Returns:
      A list of filter objects.
    """
    return [filter_class() for filter_class in cls._filter_classes.values()]

  @classmethod
  def RegisterFilter(cls, filter_class):
    """Registers a filter class.

    The filter classes are identified based on their lower case filter name.

    Args:
      filter_class: the class object of the filter.

    Raises:
      KeyError: if filter class is already set for the corresponding
                filter name.
    """
    filter_name = filter_class.__name__
    if filter_name in cls._filter_classes:
      raise KeyError((
          u'Filter class already set for filter name: {0:s}.').format(
              filter_class.__name__))

    cls._filter_classes[filter_name] = filter_class

  @classmethod
  def RegisterFilters(cls, filter_classes):
    """Registers filter classes.

    The filter classes are identified based on their lower case filter name.

    Args:
      filter_classes: a list of class objects of the filters.

    Raises:
      KeyError: if filter class is already set for the corresponding
                filter name.
    """
    for filter_class in filter_classes:
      cls.RegisterFilter(filter_class)



# -*- coding: utf-8 -*-
"""This file contains the MountPoints2 plugin."""

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import eventdata
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface


class MountPoints2Plugin(interface.WindowsRegistryPlugin):
  """Windows Registry plugin for parsing the MountPoints2 key."""

  NAME = u'explorer_mountpoints2'
  DESCRIPTION = u'Parser for mount points Registry data.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
          u'Explorer\\MountPoints2')])

  URLS = [u'http://support.microsoft.com/kb/932463']

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    for subkey in registry_key.GetSubkeys():
      name = subkey.name
      if not name:
        continue

      values_dict = {}
      values_dict[u'Volume'] = name

      label_value = subkey.GetValueByName(u'_LabelFromReg')
      if label_value:
        values_dict[u'Label'] = label_value.GetDataAsObject()

      if name.startswith(u'{'):
        values_dict[u'Type'] = u'Volume'

      elif name.startswith(u'#'):
        # The format is: ##Server_Name#Share_Name.
        values_dict[u'Type'] = u'Remote Drive'
        server_name, _, share_name = name[2:].partition(u'#')
        values_dict[u'Remote_Server'] = server_name
        values_dict[u'Share_Name'] = u'\\{0:s}'.format(
            share_name.replace(u'#', u'\\'))

      else:
        values_dict[u'Type'] = u'Drive'

      event_data = windows_events.WindowsRegistryEventData()
      event_data.key_path = registry_key.path
      event_data.offset = subkey.offset
      event_data.regvalue = values_dict
      event_data.urls = self.URLS

      event = time_events.DateTimeValuesEvent(
          subkey.last_written_time, eventdata.EventTimestamp.WRITTEN_TIME)
      parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(MountPoints2Plugin)



# -*- coding: utf-8 -*-
"""This file contains an import statement for each argument helper."""

from plaso.cli.helpers import dynamic_output
from plaso.cli.helpers import elastic_output
from plaso.cli.helpers import mysql_4n6time_output
from plaso.cli.helpers import nsrlsvr_analysis
from plaso.cli.helpers import sqlite_4n6time_output
from plaso.cli.helpers import tagging_analysis
from plaso.cli.helpers import timesketch_output
from plaso.cli.helpers import viper_analysis
from plaso.cli.helpers import virustotal_analysis
from plaso.cli.helpers import windows_services_analysis
from plaso.cli.helpers import xlsx_output



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the analysis tool object."""

import argparse
import unittest

from plaso.cli import analysis_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class StorageMediaToolTest(test_lib.CLIToolTestCase):
  """Tests for the analysis tool object."""

  _EXPECTED_OUTPUT_STORAGE_FILE_OPTIONS = u'\n'.join([
      u'usage: analysis_tool_test.py [STORAGE_FILE]',
      u'',
      u'Test argument parser.',
      u'',
      u'positional arguments:',
      u'  STORAGE_FILE  The path of the storage file.',
      u''])

  def testAddStorageFileOptions(self):
    """Tests the AddStorageFileOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'analysis_tool_test.py', description=u'Test argument parser.',
        add_help=False, formatter_class=test_lib.SortedArgumentsHelpFormatter)

    test_tool = analysis_tool.AnalysisTool()
    test_tool.AddStorageFileOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT_STORAGE_FILE_OPTIONS)

  @shared_test_lib.skipUnlessHasTestFile([u'psort_test.json.plaso'])
  def testParseOptions(self):
    """Tests the ParseOptions function."""
    test_tool = analysis_tool.AnalysisTool()

    options = test_lib.TestOptions()

    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    options.storage_file = self._GetTestFilePath([u'psort_test.json.plaso'])

    test_tool.ParseOptions(options)

    # TODO: improve this test.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The extraction and analysis CLI tool."""

import datetime
import os

from plaso.cli import status_view_tool
from plaso.lib import errors


class ExtractionAndAnalysisTool(status_view_tool.StatusViewTool):
  """Class that implements a combined extraction and analysis CLI tool."""

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (InputReader): the input reader, where None represents stdin.
      output_writer (OutputWriter): the output writer, where None represents
          stdout.
    """
    super(ExtractionAndAnalysisTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._storage_file_path = None

  def _GenerateStorageFileName(self):
    """Generates a name for the storage file.

    The result use a timestamp and the basename of the source path.

    Raises:
      BadConfigOption: raised if the source path is not set.
    """
    if not self._source_path:
      raise errors.BadConfigOption(u'Please define a source (--source).')

    timestamp = datetime.datetime.now()
    datetime_string = timestamp.strftime(u'%Y%m%dT%H%M%S')

    source_path = os.path.abspath(self._source_path)
    source_name = os.path.basename(source_path)

    if source_path.endswith(os.path.sep):
      source_path = os.path.dirname(source_path)

    source_name = os.path.basename(source_path)

    if not source_name or source_name in (u'/', u'\\'):
      # The user passed the filesystem's root as source
      source_name = u'ROOT'

    return u'{0:s}-{1:s}.plaso'.format(datetime_string, source_name)

  def _ParseStorageFileOptions(self, options):
    """Parses the storage file options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._storage_file_path = self.ParseStringOption(options, u'storage_file')
    if not self._storage_file_path:
      self._storage_file_path = self._GenerateStorageFileName()

  def AddStorageFileOptions(self, argument_group):
    """Adds the storage file options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup or argparse.ArgumentParser):
          argument group or argument parser.
    """
    argument_group.add_argument(
        u'--storage_file', action=u'store', metavar=u'STORAGE_FILE', nargs=u'?',
        type=str, default=None, help=u'The path of the storage file.')

  def ParseOptions(self, options):
    """Parses tool specific options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    super(ExtractionAndAnalysisTool, self).ParseOptions(options)
    self._ParseStorageFileOptions(options)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The image export front-end."""

import argparse
import logging
import os
import sys
import textwrap

from plaso.cli import storage_media_tool
from plaso.frontend import image_export
from plaso.lib import errors


class ImageExportTool(storage_media_tool.StorageMediaTool):
  """Class that implements the image export CLI tool.

  Attributes:
    has_filters (bool): True if filters have been specified via the options.
    list_signature_identifiers (bool): True if information about the signature
        identifiers should be shown.
  """

  NAME = u'image_export'
  DESCRIPTION = (
      u'This is a simple collector designed to export files inside an '
      u'image, both within a regular RAW image as well as inside a VSS. '
      u'The tool uses a collection filter that uses the same syntax as a '
      u'targeted plaso filter.')

  EPILOG = u'And that is how you export files, plaso style.'

  _SOURCE_OPTION = u'image'

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(ImageExportTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._destination_path = None
    self._filter_file = None
    self._front_end = image_export.ImageExportFrontend()
    self._skip_duplicates = True
    self.has_filters = False
    self.list_signature_identifiers = False

  def ListSignatureIdentifiers(self):
    """Lists the signature identifier.

    Raises:
      BadConfigOption: if the data location is invalid.
    """
    if not self._data_location:
      raise errors.BadConfigOption(u'Missing data location.')

    path = os.path.join(self._data_location, u'signatures.conf')
    if not os.path.exists(path):
      raise errors.BadConfigOption(
          u'No such format specification file: {0:s}'.format(path))

    try:
      specification_store = self._front_end.ReadSpecificationFile(path)
    except IOError as exception:
      raise errors.BadConfigOption((
          u'Unable to read format specification file: {0:s} with error: '
          u'{1:s}').format(path, exception))

    identifiers = []
    for format_specification in specification_store.specifications:
      identifiers.append(format_specification.identifier)

    self._output_writer.Write(u'Available signature identifiers:\n')
    self._output_writer.Write(
        u'\n'.join(textwrap.wrap(u', '.join(sorted(identifiers)), 79)))
    self._output_writer.Write(u'\n\n')

  def ParseArguments(self):
    """Parses the command line arguments.

    Returns:
      bool: True if the arguments were successfully parsed.
    """
    self._ConfigureLogging()

    argument_parser = argparse.ArgumentParser(
        description=self.DESCRIPTION, epilog=self.EPILOG, add_help=False,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    self.AddBasicOptions(argument_parser)
    self.AddInformationalOptions(argument_parser)
    self.AddDataLocationOption(argument_parser)
    self.AddLogFileOptions(argument_parser)

    argument_parser.add_argument(
        u'-w', u'--write', action=u'store', dest=u'path', type=str,
        metavar=u'PATH', default=u'export', help=(
            u'The directory in which extracted files should be stored.'))

    self.AddFilterOptions(argument_parser)
    argument_parser.add_argument(
        u'--date-filter', u'--date_filter', action=u'append', type=str,
        dest=u'date_filters', metavar=u'TYPE_START_END', default=[], help=(
            u'Filter based on file entry date and time ranges. This parameter '
            u'is formatted as "TIME_VALUE,START_DATE_TIME,END_DATE_TIME" where '
            u'TIME_VALUE defines which file entry timestamp the filter applies '
            u'to e.g. atime, ctime, crtime, bkup, etc. START_DATE_TIME and '
            u'END_DATE_TIME define respectively the start and end of the date '
            u'time range. A date time range requires at minimum start or end '
            u'to time of the boundary and END defines the end time. Both '
            u'timestamps be set. The date time values are formatted as: '
            u'YYYY-MM-DD hh:mm:ss.######[+-]##:## Where # are numeric digits '
            u'ranging from 0 to 9 and the seconds fraction can be either 3 '
            u'or 6 digits. The time of day, seconds fraction and timezone '
            u'offset are optional. The default timezone is UTC. E.g. "atime, '
            u'2013-01-01 23:12:14, 2013-02-23". This parameter can be repeated '
            u'as needed to add additional date date boundaries, e.g. once for '
            u'atime, once for crtime, etc.'))

    argument_parser.add_argument(
        u'-x', u'--extensions', dest=u'extensions_string', action=u'store',
        type=str, metavar=u'EXTENSIONS', help=(
            u'Filter based on file name extensions. This option accepts '
            u'multiple multiple comma separated values e.g. "csv,docx,pst".'))

    argument_parser.add_argument(
        u'--names', dest=u'names_string', action=u'store',
        type=str, metavar=u'NAMES', help=(
            u'If the purpose is to find all files given a certain names '
            u'this options should be used. This option accepts a comma '
            u'separated string denoting all file names, e.g. -x '
            u'"NTUSER.DAT,UsrClass.dat".'))

    argument_parser.add_argument(
        u'--signatures', dest=u'signature_identifiers', action=u'store',
        type=str, metavar=u'IDENTIFIERS', help=(
            u'Filter based on file format signature identifiers. This option '
            u'accepts multiple comma separated values e.g. "esedb,lnk". '
            u'Use "list" to show an overview of the supported file format '
            u'signatures.'))

    argument_parser.add_argument(
        u'--include_duplicates', dest=u'include_duplicates',
        action=u'store_true', default=False, help=(
            u'If extraction from VSS is enabled, by default a digest hash '
            u'is calculated for each file. These hashes are compared to the '
            u'previously exported files and duplicates are skipped. Use '
            u'this option to include duplicate files in the export.'))

    self.AddStorageMediaImageOptions(argument_parser)
    self.AddVSSProcessingOptions(argument_parser)

    argument_parser.add_argument(
        self._SOURCE_OPTION, nargs='?', action=u'store', metavar=u'IMAGE',
        default=None, type=str, help=(
            u'The full path to the image file that we are about to extract '
            u'files from, it should be a raw image or another image that '
            u'plaso supports.'))

    try:
      options = argument_parser.parse_args()
    except UnicodeEncodeError:
      # If we get here we are attempting to print help in a non-Unicode
      # terminal.
      self._output_writer.Write(u'')
      self._output_writer.Write(argument_parser.format_help())
      return False

    try:
      self.ParseOptions(options)
    except errors.BadConfigOption as exception:
      self._output_writer.Write(u'ERROR: {0:s}'.format(exception))
      self._output_writer.Write(u'')
      self._output_writer.Write(argument_parser.format_usage())
      return False

    return True

  def ParseOptions(self, options):
    """Parses the options and initializes the front-end.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    # The data location is required to list signatures.
    self._ParseDataLocationOption(options)

    # Check the list options first otherwise required options will raise.
    signature_identifiers = self.ParseStringOption(
        options, u'signature_identifiers')
    if signature_identifiers == u'list':
      self.list_signature_identifiers = True

    if self.list_signature_identifiers:
      return

    super(ImageExportTool, self).ParseOptions(options)

    format_string = (
        u'%(asctime)s [%(levelname)s] (%(processName)-10s) PID:%(process)d '
        u'<%(module)s> %(message)s')

    if self._debug_mode:
      logging_level = logging.DEBUG
    elif self._quiet_mode:
      logging_level = logging.WARNING
    else:
      logging_level = logging.INFO

    self.ParseLogFileOptions(options)
    self._ConfigureLogging(
        filename=self._log_file, format_string=format_string,
        log_level=logging_level)

    self._destination_path = self.ParseStringOption(
        options, u'path', default_value=u'export')

    self._ParseFilterOptions(options)

    if (getattr(options, u'no_vss', False) or
        getattr(options, u'include_duplicates', False)):
      self._skip_duplicates = False

    date_filters = getattr(options, u'date_filters', None)
    try:
      self._front_end.ParseDateFilters(date_filters)
    except ValueError as exception:
      raise errors.BadConfigOption(exception)

    extensions_string = self.ParseStringOption(options, u'extensions_string')
    self._front_end.ParseExtensionsString(extensions_string)

    names_string = getattr(options, u'names_string', None)
    self._front_end.ParseNamesString(names_string)

    if not self._data_location:
      logging.warning(u'Unable to automatically determine data location.')

    signature_identifiers = getattr(options, u'signature_identifiers', None)
    try:
      self._front_end.ParseSignatureIdentifiers(
          self._data_location, signature_identifiers)
    except (IOError, ValueError) as exception:
      raise errors.BadConfigOption(exception)

    if self._filter_file:
      self.has_filters = True
    else:
      self.has_filters = self._front_end.HasFilters()

  def PrintFilterCollection(self):
    """Prints the filter collection."""
    self._front_end.PrintFilterCollection(self._output_writer)

  def ProcessSources(self):
    """Processes the sources.

    Raises:
      SourceScannerError: if the source scanner could not find a supported
                          file system.
      UserAbort: if the user initiated an abort.
    """
    self.ScanSource()

    self._output_writer.Write(u'Export started.\n')

    self._front_end.ProcessSources(
        self._source_path_specs, self._destination_path, self._output_writer,
        filter_file=self._filter_file, skip_duplicates=self._skip_duplicates)

    self._output_writer.Write(u'Export completed.\n')
    self._output_writer.Write(u'\n')


def Main():
  """The main function."""
  tool = ImageExportTool()

  if not tool.ParseArguments():
    return False

  if tool.list_signature_identifiers:
    tool.ListSignatureIdentifiers()
    return True

  if not tool.has_filters:
    logging.warning(u'No filter defined exporting all files.')

  # TODO: print more status information like PrintOptions.
  tool.PrintFilterCollection()

  try:
    tool.ProcessSources()

  except (KeyboardInterrupt, errors.UserAbort):
    logging.warning(u'Aborted by user.')
    return False

  except errors.BadConfigOption as exception:
    logging.warning(exception)
    return False

  except errors.SourceScannerError as exception:
    logging.warning((
        u'Unable to scan for a supported filesystem with error: {0:s}\n'
        u'Most likely the image format is not supported by the '
        u'tool.').format(exception))
    return False

  return True


if __name__ == '__main__':
  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



# -*- coding: utf-8 -*-
"""This file contains the airport plist plugin in Plaso."""

from plaso.containers import plist_event
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class AirportPlugin(interface.PlistPlugin):
  """Plist plugin that extracts WiFi information."""

  NAME = u'airport'
  DESCRIPTION = u'Parser for Airport plist files.'

  PLIST_PATH = u'com.apple.airport.preferences.plist'
  PLIST_KEYS = frozenset([u'RememberedNetworks'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant Airport entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    if u'RememberedNetworks' not in match:
      return

    for wifi in match[u'RememberedNetworks']:
      description = (
          u'[WiFi] Connected to network: <{0:s}> using security {1:s}').format(
              wifi.get(u'SSIDString', u'UNKNOWN_SSID'),
              wifi.get(u'SecurityType', u'UNKNOWN_SECURITY_TYPE'))
      event_object = plist_event.PlistEvent(
          u'/RememberedNetworks', u'item', wifi.get(u'LastConnected', 0),
          description)
      parser_mediator.ProduceEvent(event_object)


plist.PlistParser.RegisterPlugin(AirportPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the tagging analysis plugin."""

import unittest

from plaso.analysis import tagging
from plaso.lib import timelib
from plaso.containers import events

from tests import test_lib as shared_test_lib
from tests.analysis import test_lib


class TestPrefetchEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = u'windows:prefetch'


class TestChromeDownloadEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = u'chrome:history:file_downloaded'


class TestEvtRecordEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = u'windows:evt:record'


class TaggingAnalysisPluginTest(test_lib.AnalysisPluginTestCase):
  """Tests the tagging analysis plugin."""

  # pylint: disable=protected-access

  _INVALID_TEST_TAG_FILE_NAME = u'invalid_test_tag_file.txt'
  _TEST_TAG_FILE_NAME = u'test_tag_file.txt'

  _TEST_EVENTS = [
      {u'event_type': u'prefetch',
       u'timestamp': timelib.Timestamp.CopyFromString(u'2015-05-01 15:12:00'),
       u'attributes': {}
      },
      {u'event_type': u'chrome_download',
       u'timestamp': timelib.Timestamp.CopyFromString(u'2015-05-01 05:06:00'),
       u'attributes': {}
      },
      {u'event_type': u'something_else',
       u'timestamp': timelib.Timestamp.CopyFromString(u'2015-02-19 08:00:01'),
       u'attributes': {}
      },
      {u'event_type': u'evt',
       u'timestamp': timelib.Timestamp.CopyFromString(u'2016-05-25 13:00:06'),
       u'attributes': {
           u'source_name': u'Security',
           u'event_identifier': 538}
      },
      {u'event_type': u'evt',
       u'timestamp': timelib.Timestamp.CopyFromString(u'2016-05-25 13:00:06'),
       u'attributes': {
           u'source_name': u'Messaging',
           u'event_identifier': 16,
           u'body': u'this is a message'}
      },
  ]

  def _CreateTestEventObject(self, event_attributes):
    """Create a test event with a set of attributes.

    Args:
      event_attributes (dict[str, str]): attributes of an event to add to the
          queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    if event_attributes[u'event_type'] == u'prefetch':
      event_object = TestPrefetchEvent()
    elif event_attributes[u'event_type'] == u'chrome_download':
      event_object = TestChromeDownloadEvent()
    elif event_attributes[u'event_type'] == u'evt':
      event_object = TestEvtRecordEvent()
    else:
      event_object = events.EventObject()

    event_object.timestamp = event_attributes[u'timestamp']
    for key, value in iter(event_attributes[u'attributes'].items()):
      setattr(event_object, key, value)
    return event_object

  @shared_test_lib.skipUnlessHasTestFile([u'test_tag_file.txt'])
  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    test_events = []
    for event_dictionary in self._TEST_EVENTS:
      event = self._CreateTestEventObject(event_dictionary)
      test_events.append(event)

    test_file = self._GetTestFilePath([self._TEST_TAG_FILE_NAME])
    plugin = tagging.TaggingAnalysisPlugin()
    plugin.SetAndLoadTagFile(test_file)

    storage_writer = self._AnalyzeEvents(test_events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(len(storage_writer.event_tags), 4)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = u'Tagging plugin produced 4 tags.\n'
    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.event_tags:
      labels.extend(event_tag.labels)

    self.assertEqual(len(labels), 5)

    # This is from a tag rule declared in objectfilter syntax.
    self.assertIn(u'application_execution', labels)
    # This is from a tag rule declared in dotty syntax.
    self.assertIn(u'login_attempt', labels)
    # This is from a rule using the "contains" operator
    self.assertIn(u'text_contains', labels)

  @shared_test_lib.skipUnlessHasTestFile([u'test_tag_file.txt'])
  @shared_test_lib.skipUnlessHasTestFile([u'invalid_test_tag_file.txt'])
  def testParseTaggingFile(self):
    """Tests the _ParseTaggingFile function."""
    plugin = tagging.TaggingAnalysisPlugin()
    test_path = self._GetTestFilePath([self._TEST_TAG_FILE_NAME])

    tag_expression = plugin._ParseTaggingFile(test_path)
    self.assertEqual(len(tag_expression.children), 5)

    plugin = tagging.TaggingAnalysisPlugin()
    test_path = self._GetTestFilePath([self._INVALID_TEST_TAG_FILE_NAME])

    tag_expression = plugin._ParseTaggingFile(test_path)
    self.assertEqual(len(tag_expression.children), 2)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the parsers manager."""

import unittest

from plaso.parsers import interface
from plaso.parsers import manager
from plaso.parsers import plugins

from tests import test_lib as shared_test_lib


class TestParser(interface.BaseParser):
  """Test parser."""

  NAME = u'test_parser'
  DESCRIPTION = u'Test parser.'

  # pylint: disable=unused-argument
  def Parse(self, parser_mediator, **kwargs):
    """Parses the file entry and extracts event objects.

    Args:
      parser_mediator (ParserMediator): parser mediator.
    """
    return


class TestParserWithPlugins(interface.BaseParser):
  """Test parser with plugins."""

  NAME = u'test_parser_with_plugins'
  DESCRIPTION = u'Test parser with plugins.'

  _plugin_classes = {}

  # pylint: disable=unused-argument
  def Parse(self, parser_mediator, **kwargs):
    """Parses the file entry and extracts event objects.

    Args:
      parser_mediator (ParserMediator): parser mediator.
    """
    return


class TestPlugin(plugins.BasePlugin):
  """Test plugin."""

  NAME = u'test_plugin'
  DESCRIPTION = u'Test plugin.'

  # pylint: disable=unused-argument
  def Process(self, parser_mediator, **kwargs):
    """Evaluates if this is the correct plugin and processes data accordingly.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      kwargs: Depending on the plugin they may require different sets of
              arguments to be able to evaluate whether or not this is
              the correct plugin.

    Raises:
      ValueError: When there are unused keyword arguments.
    """
    return


class ParsersManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the parsers manager."""

  # pylint: disable=protected-access

  def testGetParserFilters(self):
    """Tests the _GetParserFilters function."""
    parser_filter_expression = u''
    includes, excludes = manager.ParsersManager._GetParserFilters(
        parser_filter_expression)
    self.assertEqual(includes, {})
    self.assertEqual(excludes, {})

    parser_filter_expression = u'test_include,!test_exclude'
    includes, excludes = manager.ParsersManager._GetParserFilters(
        parser_filter_expression)
    self.assertEqual(includes, {u'test_include': []})
    self.assertEqual(excludes, {})

    parser_filter_expression = (
        u'test_include,test_intersection,!test_exclude,!test_intersection')
    includes, excludes = manager.ParsersManager._GetParserFilters(
        parser_filter_expression)
    self.assertEqual(includes, {u'test_include': []})
    self.assertEqual(excludes, {})

    parser_filter_expression = u'test/include,!test/exclude'
    includes, excludes = manager.ParsersManager._GetParserFilters(
        parser_filter_expression)
    self.assertEqual(includes, {u'test': [u'include']})
    self.assertEqual(excludes, {u'test': [u'exclude']})

    parser_filter_expression = (
        u'test/include,test/intersection,!test/exclude,!test/intersection')
    includes, excludes = manager.ParsersManager._GetParserFilters(
        parser_filter_expression)
    self.assertEqual(includes, {u'test': [u'include']})
    self.assertEqual(excludes, {u'test': [u'exclude', u'intersection']})

  def testGetParsersFromPresetCategory(self):
    """Tests the _GetParsersFromPresetCategory function."""
    expected_parser_names = [
        u'bencode', u'binary_cookies', u'chrome_cache', u'chrome_preferences',
        u'esedb', u'esedb/msie_webcache', u'filestat', u'firefox_cache',
        u'java_idx', u'lnk', u'mcafee_protection', u'msiecf', u'olecf',
        u'openxml', u'opera_global', u'opera_typed_history', u'pe',
        u'plist/safari_history', u'prefetch', u'sccm', u'skydrive_log',
        u'skydrive_log_old', u'sqlite/chrome_cookies',
        u'sqlite/chrome_extension_activity', u'sqlite/chrome_history',
        u'sqlite/firefox_cookies', u'sqlite/firefox_downloads',
        u'sqlite/firefox_history', u'sqlite/google_drive', u'sqlite/skype',
        u'symantec_scanlog', u'winfirewall', u'winjob', u'winreg']

    parser_names = manager.ParsersManager._GetParsersFromPresetCategory(
        u'win_gen')
    self.assertEqual(parser_names, expected_parser_names)

    parser_names = manager.ParsersManager._GetParsersFromPresetCategory(
        u'bogus')
    self.assertEqual(parser_names, [])

  def testReduceParserFilters(self):
    """Tests the ReduceParserFilters function."""
    includes = {}
    excludes = {}

    manager.ParsersManager._ReduceParserFilters(includes, excludes)
    self.assertEqual(includes, {})
    self.assertEqual(excludes, {})

    includes = {u'test_include': u''}
    excludes = {u'test_exclude': u''}

    manager.ParsersManager._ReduceParserFilters(includes, excludes)
    self.assertEqual(includes, {u'test_include': u''})
    self.assertEqual(excludes, {})

    includes = {u'test_include': u'', u'test_intersection': u''}
    excludes = {u'test_exclude': u'', u'test_intersection': u''}

    manager.ParsersManager._ReduceParserFilters(includes, excludes)
    self.assertEqual(includes, {u'test_include': u''})
    self.assertEqual(excludes, {})

    includes = {u'test': [u'include']}
    excludes = {u'test': [u'exclude']}

    manager.ParsersManager._ReduceParserFilters(includes, excludes)
    self.assertEqual(includes, {u'test': [u'include']})
    self.assertEqual(excludes, {u'test': [u'exclude']})

    includes = {u'test': [u'include', u'intersection']}
    excludes = {u'test': [u'exclude', u'intersection']}

    manager.ParsersManager._ReduceParserFilters(includes, excludes)
    self.assertEqual(includes, {u'test': [u'include']})
    self.assertEqual(excludes, {u'test': [u'exclude', u'intersection']})

  def testParserRegistration(self):
    """Tests the RegisterParser and DeregisterParser functions."""
    number_of_parsers = len(manager.ParsersManager._parser_classes)

    manager.ParsersManager.RegisterParser(TestParser)
    self.assertEqual(
        len(manager.ParsersManager._parser_classes),
        number_of_parsers + 1)

    with self.assertRaises(KeyError):
      manager.ParsersManager.RegisterParser(TestParser)

    manager.ParsersManager.DeregisterParser(TestParser)
    self.assertEqual(
        len(manager.ParsersManager._parser_classes),
        number_of_parsers)

  def testPluginRegistration(self):
    """Tests the RegisterPlugin and DeregisterPlugin functions."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    self.assertEqual(
        len(TestParserWithPlugins._plugin_classes), 1)

    with self.assertRaises(KeyError):
      TestParserWithPlugins.RegisterPlugin(TestPlugin)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    self.assertEqual(
        len(TestParserWithPlugins._plugin_classes), 0)

  def testGetParserAndPluginNames(self):
    """Tests the GetParserAndPluginNames function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression=u'test_parser')
    self.assertEqual(parser_names, [u'test_parser'])

    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression=u'!test_parser')
    self.assertNotIn(u'test_parser', parser_names)

    expected_parser_names = [
        u'test_parser_with_plugins',
        u'test_parser_with_plugins/test_plugin']
    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression=u'test_parser_with_plugins/test_plugin')
    self.assertEqual(parser_names, expected_parser_names)

    # Test with a parser name, not using plugin names.
    expected_parser_names = [
        u'test_parser_with_plugins',
        u'test_parser_with_plugins/test_plugin']
    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression=u'test_parser_with_plugins')
    self.assertEqual(parser_names, expected_parser_names)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

    # Test with a preset name.
    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression=u'win_gen')
    self.assertIn(u'lnk', parser_names)

  def testGetParserObjectByName(self):
    """Tests the GetParserObjectByName function."""
    manager.ParsersManager.RegisterParser(TestParser)

    parser_object = manager.ParsersManager.GetParserObjectByName(
        u'test_parser')
    self.assertIsNotNone(parser_object)
    self.assertEqual(parser_object.NAME, u'test_parser')

    parser_object = manager.ParsersManager.GetParserObjectByName(u'bogus')
    self.assertIsNone(parser_object)

    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParserObjects(self):
    """Tests the GetParserObjects function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = []
    parser_objects = manager.ParsersManager.GetParserObjects(
        parser_filter_expression=u'test_parser')
    for _, parser_object in iter(parser_objects.items()):
      parser_names.append(parser_object.NAME)
    self.assertEqual(parser_names, [u'test_parser'])

    parser_names = []
    parser_objects = manager.ParsersManager.GetParserObjects(
        parser_filter_expression=u'!test_parser')
    for _, parser_object in iter(parser_objects.items()):
      parser_names.append(parser_object.NAME)
    self.assertNotEqual(len(parser_names), 0)
    self.assertNotIn(u'test_parser', parser_names)

    parser_names = []
    parser_objects = manager.ParsersManager.GetParserObjects(
        parser_filter_expression=u'test_parser_with_plugins/test_plugin')
    for _, parser_object in iter(parser_objects.items()):
      parser_names.append(parser_object.NAME)
    self.assertEqual(parser_names, [u'test_parser_with_plugins'])

    # Test with a parser name, not using plugin names.
    parser_names = []
    parser_objects = manager.ParsersManager.GetParserObjects(
        parser_filter_expression=u'test_parser_with_plugins')
    for _, parser_object in iter(parser_objects.items()):
      parser_names.append(parser_object.NAME)
    self.assertEqual(parser_names, [u'test_parser_with_plugins'])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParsers(self):
    """Tests the GetParsers function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = []
    for _, parser_class in manager.ParsersManager.GetParsers(
        parser_filter_expression=u'test_parser'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, [u'test_parser'])

    parser_names = []
    for _, parser_class in manager.ParsersManager.GetParsers(
        parser_filter_expression=u'!test_parser'):
      parser_names.append(parser_class.NAME)
    self.assertNotEqual(len(parser_names), 0)
    self.assertNotIn(u'test_parser', parser_names)

    parser_names = []
    for _, parser_class in manager.ParsersManager.GetParsers(
        parser_filter_expression=u'test_parser_with_plugins/test_plugin'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, [u'test_parser_with_plugins'])

    # Test with a parser name, not using plugin names.
    parser_names = []
    for _, parser_class in manager.ParsersManager.GetParsers(
        parser_filter_expression=u'test_parser_with_plugins'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, [u'test_parser_with_plugins'])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

    # Test with a preset name.
    expected_parser_names = [
        u'bencode', u'binary_cookies', u'chrome_cache', u'chrome_preferences',
        u'esedb', u'filestat', u'firefox_cache', u'java_idx', u'lnk',
        u'mcafee_protection', u'msiecf', u'olecf', u'openxml', u'opera_global',
        u'opera_typed_history', u'pe', u'plist', u'prefetch', u'sccm',
        u'skydrive_log', u'skydrive_log_old', u'sqlite', u'symantec_scanlog',
        u'winfirewall', u'winjob', u'winreg']

    parser_names = []
    for _, parser_class in manager.ParsersManager.GetParsers(
        parser_filter_expression=u'win_gen'):
      parser_names.append(parser_class.NAME)

    self.assertEqual(sorted(parser_names), expected_parser_names)

  def testGetPluginObjectByName(self):
    """Tests the GetPluginObjectByName function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)

    plugin_object = TestParserWithPlugins.GetPluginObjectByName(u'test_plugin')
    self.assertIsNotNone(plugin_object)

    plugin_object = TestParserWithPlugins.GetPluginObjectByName(u'bogus')
    self.assertIsNone(plugin_object)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)

  def testGetPlugins(self):
    """Tests the GetPlugins function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)

    generator = TestParserWithPlugins.GetPlugins()
    plugin_tuples = list(generator)
    self.assertNotEqual(len(plugin_tuples), 0)
    self.assertIsNotNone(plugin_tuples[0])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)

  # TODO: add GetParsersInformation test.
  # TODO: add GetNamesOfParsersWithPlugins test.
  # TODO: add GetScanner test.
  # TODO: add GetSpecificationStore test.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains classes used for preprocessing in plaso."""

import abc

from dfvfs.helpers import file_system_searcher

from plaso.containers import artifacts
from plaso.lib import errors
from plaso.lib import py2to3


class PreprocessPlugin(object):
  """The preprocess plugin interface."""

  @property
  def plugin_name(self):
    """str: name of the plugin."""
    return self.__class__.__name__


class FileSystemPreprocessPlugin(PreprocessPlugin):
  """The file system preprocess plugin interface."""

  def _FindFileEntry(self, searcher, path):
    """Searches for a file entry that matches the path.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.
      path (str): location of the file entry relative to the file system
          of the searcher.

    Returns:
      dfvfs.FileEntry: file entry if successful or None otherwise.

    Raises:
      errors.PreProcessFail: if the file entry cannot be opened.
    """
    path_specs = self._FindPathSpecs(searcher, path)
    if not path_specs or len(path_specs) != 1:
      return

    try:
      return searcher.GetFileEntryByPathSpec(path_specs[0])
    except IOError as exception:
      raise errors.PreProcessFail(
          u'Unable to retrieve file entry: {0:s} with error: {1:s}'.format(
              path, exception))

  def _FindPathSpecs(self, searcher, path):
    """Searches for path specifications that matches the path.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.
      path (str): location of the file entry relative to the file system
          of the searcher.

    Returns:
      list[dfvfs.PathSpec]: path specifcations.
    """
    find_spec = file_system_searcher.FindSpec(
        location_regex=path, case_sensitive=False)
    return list(searcher.Find(find_specs=[find_spec]))

  @abc.abstractmethod
  def Run(self, searcher, knowledge_base):
    """Determines the value of the preprocessing attributes.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
    """


class FilePreprocessPlugin(FileSystemPreprocessPlugin):
  """The file preprocess plugin interface."""

  _PATH = None

  @abc.abstractmethod
  def _ParseFileObject(self, knowledge_base, file_object):
    """Parses a file-like object.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_object (dfvfs.FileIO): file-like object.
    """

  def Run(self, searcher, knowledge_base):
    """Determines the value of the preprocessing attributes.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
    """
    file_entry = self._FindFileEntry(searcher, self._PATH)
    if not file_entry:
      return

    file_object = file_entry.GetFileObject()
    try:
      self._ParseFileObject(knowledge_base, file_object)
    finally:
      file_object.close()


class WindowsPathEnvironmentVariablePlugin(FileSystemPreprocessPlugin):
  """Windows path environment variable preprocess plugin interface."""

  _NAME = None
  _PATH_REGEX = None

  def Run(self, searcher, knowledge_base):
    """Determines the value of the preprocessing attributes.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    path_specs = self._FindPathSpecs(searcher, self._PATH_REGEX)
    if not path_specs:
      return

    relative_path = searcher.GetRelativePath(path_specs[0])
    if not relative_path:
      raise errors.PreProcessFail(
          u'Missing relative path for: {0:s}'.format(self._PATH_REGEX))

    if relative_path.startswith(u'/'):
      path_segments = relative_path.split(u'/')
      relative_path = u'\\'.join(path_segments)

    evironment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=relative_path)

    try:
      knowledge_base.AddEnvironmentVariable(evironment_variable)
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass


class WindowsRegistryKeyPreprocessPlugin(PreprocessPlugin):
  """Windows Registry key preprocess plugin interface."""

  _REGISTRY_KEY_PATH = None

  @abc.abstractmethod
  def _ParseKey(self, knowledge_base, registry_key):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (WinRegistryKey): Windows Registry key.
    """

  def Run(self, win_registry, knowledge_base):
    """Determines the value of the preprocessing attributes.

    Args:
      win_registry (WinRegistry): Windows Registry.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.

    Raises:
      PreProcessFail: if there was an error retrieving the Registry key.
    """
    try:
      registry_key = win_registry.GetKeyByPath(self._REGISTRY_KEY_PATH)

    except IOError as exception:
      raise errors.PreProcessFail(
          u'Unable to retrieve Registry key: {0:s} with error: {1:s}'.format(
              self._REGISTRY_KEY_PATH, exception))

    if registry_key:
      self._ParseKey(knowledge_base, registry_key)


class WindowsRegistryValuePreprocessPlugin(WindowsRegistryKeyPreprocessPlugin):
  """Windows Registry value preprocess plugin interface."""

  _REGISTRY_VALUE_NAME = u''

  def _ParseKey(self, knowledge_base, registry_key):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (WinRegistryKey): Windows Registry key.

    Raises:
      errors.PreProcessFail: if the Registry value or value data can not be
          retrieved.
    """
    try:
      registry_value = registry_key.GetValueByName(self._REGISTRY_VALUE_NAME)
    except IOError as exception:
      raise errors.PreProcessFail((
          u'Unable to retrieve Registry key: {0:s}, value: {1:s} with '
          u'error: {2:s}').format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME, exception))

    if not registry_value:
      return

    try:
      value_data = registry_value.GetDataAsObject()
    except IOError as exception:
      raise errors.PreProcessFail((
          u'Unable to retrieve Registry key: {0:s}, value: {1:s} data with '
          u'error: {2:s}').format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME, exception))

    if not value_data:
      return

    self._ParseValueData(knowledge_base, value_data)

  @abc.abstractmethod
  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.
    """


class WindowsRegistryEnvironmentVariable(WindowsRegistryValuePreprocessPlugin):
  """Windows Registry environment variable preprocess plugin interface."""

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not supported.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME))

    evironment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=value_data)

    try:
      knowledge_base.AddEnvironmentVariable(evironment_variable)
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass



# -*- coding: utf-8 -*-
"""The filter interface classes."""

import abc

from plaso.lib import errors
from plaso.lib import pfilter


class FilterObject(object):
  """The filter interface class."""

  def __init__(self):
    """Initializes a filter object."""
    super(FilterObject, self).__init__()
    self._filter_expression = None
    self._matcher = None

  @property
  def fields(self):
    """Return a list of fields for adaptive output modules."""
    return []

  @property
  def filter_expression(self):
    """Return the compiled filter expression or None if not compiled."""
    if self._filter_expression:
      return self._filter_expression

  @property
  def filter_name(self):
    """Return the name of the filter."""
    return self.__class__.__name__

  @property
  def limit(self):
    """The row limit."""
    return 0

  @property
  def separator(self):
    """The output field separator value."""
    return u','

  def _GetMatcher(self, filter_expression):
    """Retrieves a filter object for a specific filter expression.

    Args:
      filter_expression: string that contains the filter expression.

    Returns:
      A filter object (instance of objectfilter.TODO) or None.
    """
    try:
      parser = pfilter.BaseParser(filter_expression).Parse()
      return parser.Compile(pfilter.PlasoAttributeFilterImplementation)

    except errors.ParseError:
      pass

  @abc.abstractmethod
  def CompileFilter(self, filter_expression):
    """Compiles the filter expression.

    Args:
      filter_expression: string that contains the filter expression.

    Raises:
      WrongPlugin: if the filter could not be compiled.
    """

  def Match(self, unused_event_object):
    """Determines if an event object matches the filter.

    Args:
      event_object: an event object (instance of EventObject).

    Returns:
      A boolean value that indicates a match.
    """
    return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Java Cache IDX file parser."""

import unittest

from plaso.formatters import java_idx  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import java_idx

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class IDXTest(test_lib.ParserTestCase):
  """Tests for Java Cache IDX file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'java_602.idx'])
  def testParse602(self):
    """Tests the Parse function on a version 602 IDX file."""
    parser_object = java_idx.JavaIDXParser()
    storage_writer = self._ParseFile([u'java_602.idx'], parser_object)

    self.assertEqual(len(storage_writer.events), 2)

    event_object = storage_writer.events[0]

    idx_version_expected = 602
    self.assertEqual(event_object.idx_version, idx_version_expected)

    ip_address_expected = u'Unknown'
    self.assertEqual(event_object.ip_address, ip_address_expected)

    url_expected = u'http://www.gxxxxx.com/a/java/xxz.jar'
    self.assertEqual(event_object.url, url_expected)

    description_expected = u'File Hosted Date'
    self.assertEqual(event_object.timestamp_desc, description_expected)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-05 01:34:19.720')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # Parse second event. Same metadata; different timestamp event.
    event_object = storage_writer.events[1]

    self.assertEqual(event_object.idx_version, idx_version_expected)
    self.assertEqual(event_object.ip_address, ip_address_expected)
    self.assertEqual(event_object.url, url_expected)

    description_expected = eventdata.EventTimestamp.FILE_DOWNLOADED
    self.assertEqual(event_object.timestamp_desc, description_expected)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-05 03:52:31')
    self.assertEqual(event_object.timestamp, expected_timestamp)

  @shared_test_lib.skipUnlessHasTestFile([u'java.idx'])
  def testParse605(self):
    """Tests the Parse function on a version 605 IDX file."""
    parser_object = java_idx.JavaIDXParser()
    storage_writer = self._ParseFile([u'java.idx'], parser_object)

    self.assertEqual(len(storage_writer.events), 2)

    event_object = storage_writer.events[0]

    idx_version_expected = 605
    self.assertEqual(event_object.idx_version, idx_version_expected)

    ip_address_expected = u'10.7.119.10'
    self.assertEqual(event_object.ip_address, ip_address_expected)

    url_expected = (
        u'http://xxxxc146d3.gxhjxxwsf.xx:82/forum/dare.php?'
        u'hsh=6&key=b30xxxx1c597xxxx15d593d3f0xxx1ab')
    self.assertEqual(event_object.url, url_expected)

    description_expected = u'File Hosted Date'
    self.assertEqual(event_object.timestamp_desc, description_expected)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2001-07-26 05:00:00')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # Parse second event. Same metadata; different timestamp event.
    event_object = storage_writer.events[1]

    self.assertEqual(event_object.idx_version, idx_version_expected)
    self.assertEqual(event_object.ip_address, ip_address_expected)
    self.assertEqual(event_object.url, url_expected)

    description_expected = eventdata.EventTimestamp.FILE_DOWNLOADED
    self.assertEqual(event_object.timestamp_desc, description_expected)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-01-13 16:22:01')
    self.assertEqual(event_object.timestamp, expected_timestamp)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Google Analytics cookies."""

import unittest

from plaso.formatters import ganalytics  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers.cookie_plugins import ganalytics
from plaso.parsers.sqlite_plugins import chrome_cookies
from plaso.parsers.sqlite_plugins import firefox_cookies

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib as sqlite_plugins_test_lib


class GoogleAnalyticsPluginTest(sqlite_plugins_test_lib.SQLitePluginTestCase):
  """Tests for the Google Analytics plugin."""

  def _GetAnalyticsCookieEvents(self, storage_writer):
    """Retrieves the analytics cookie events.

    Returns:
      list[EventObject]: analytics cookie events.
    """
    cookies = []
    for event in storage_writer.events:
      if event.data_type.startswith(u'cookie:google:analytics'):
        cookies.append(event)
    return cookies

  @shared_test_lib.skipUnlessHasTestFile([u'firefox_cookies.sqlite'])
  def testParsingFirefox29CookieDatabase(self):
    """Tests the Process function on a Firefox 29 cookie database file."""
    plugin_object = firefox_cookies.FirefoxCookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'firefox_cookies.sqlite'], plugin_object)
    test_events = self._GetAnalyticsCookieEvents(storage_writer)

    self.assertEqual(len(test_events), 25)

    event = test_events[14]

    self.assertEqual(
        event.utmcct,
        u'/frettir/erlent/2013/10/30/maelt_med_kerfisbundnum_hydingum/')
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-30 21:56:06')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.url, u'http://ads.aha.is/')
    self.assertEqual(event.utmcsr, u'mbl.is')

    expected_msg = (
        u'http://ads.aha.is/ (__utmz) Sessions: 1 Domain Hash: 137167072 '
        u'Sources: 1 Last source used to access: mbl.is Ad campaign '
        u'information: (referral) Last type of visit: referral Path to '
        u'the page of referring link: /frettir/erlent/2013/10/30/'
        u'maelt_med_kerfisbundnum_hydingum/')

    self._TestGetMessageStrings(
        event, expected_msg, u'http://ads.aha.is/ (__utmz)')

  @shared_test_lib.skipUnlessHasTestFile([u'cookies.db'])
  def testParsingChromeCookieDatabase(self):
    """Test the process function on a Chrome cookie database."""
    plugin_object = chrome_cookies.ChromeCookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'cookies.db'], plugin_object)
    test_events = self._GetAnalyticsCookieEvents(storage_writer)

    # The cookie database contains 560 entries in total. Out of them
    # there are 75 events created by the Google Analytics plugin.
    self.assertEqual(len(test_events), 75)
    # Check few "random" events to verify.

    # Check an UTMZ Google Analytics event.
    event = test_events[39]
    self.assertEqual(event.utmctr, u'enders game')
    self.assertEqual(event.domain_hash, u'68898382')
    self.assertEqual(event.sessions, 1)

    expected_msg = (
        u'http://imdb.com/ (__utmz) Sessions: 1 Domain Hash: 68898382 '
        u'Sources: 1 Last source used to access: google Ad campaign '
        u'information: (organic) Last type of visit: organic Keywords '
        u'used to find site: enders game')
    self._TestGetMessageStrings(
        event, expected_msg, u'http://imdb.com/ (__utmz)')

    # Check the UTMA Google Analytics event.
    event = test_events[41]
    self.assertEqual(event.timestamp_desc, u'Analytics Previous Time')
    self.assertEqual(event.cookie_name, u'__utma')
    self.assertEqual(event.visitor_id, u'1827102436')
    self.assertEqual(event.sessions, 2)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-22 01:55:29')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_msg = (
        u'http://assets.tumblr.com/ (__utma) '
        u'Sessions: 2 '
        u'Domain Hash: 151488169 '
        u'Visitor ID: 1827102436')
    self._TestGetMessageStrings(
        event, expected_msg, u'http://assets.tumblr.com/ (__utma)')

    # Check the UTMB Google Analytics event.
    event = test_events[34]
    self.assertEqual(
        event.timestamp_desc, eventdata.EventTimestamp.LAST_VISITED_TIME)
    self.assertEqual(event.cookie_name, u'__utmb')
    self.assertEqual(event.domain_hash, u'154523900')
    self.assertEqual(event.pages_viewed, 1)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-22 01:48:30')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_msg = (
        u'http://upressonline.com/ (__utmb) Pages Viewed: 1 Domain Hash: '
        u'154523900')
    self._TestGetMessageStrings(
        event, expected_msg, u'http://upressonline.com/ (__utmb)')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the psort front-end."""

import unittest

from tests import test_lib as shared_test_lib


class PsortFrontendTest(shared_test_lib.BaseTestCase):
  """Tests for the psort front-end."""

  # TODO: implement.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows XML EventLog (EVTX) file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinEVTXFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows XML EventLog (EVTX) record event."""

  DATA_TYPE = u'windows:evtx:record'

  FORMAT_STRING_PIECES = [
      u'[{event_identifier} /',
      u'0x{event_identifier:04x}]',
      u'Record Number: {record_number}',
      u'Event Level: {event_level}',
      u'Source Name: {source_name}',
      u'Computer Name: {computer_name}',
      u'Message string: {message_string}',
      u'Strings: {strings}']

  FORMAT_STRING_SHORT_PIECES = [
      u'[{event_identifier} /',
      u'0x{event_identifier:04x}]',
      u'Strings: {strings}']

  SOURCE_LONG = u'WinEVTX'
  SOURCE_SHORT = u'EVT'

  def GetMessages(self, formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    source_name = event_values.get(u'source_name', None)
    message_identifier = event_values.get(u'message_identifier', None)
    strings = event_values.get(u'strings', [])
    if source_name and message_identifier:
      message_string = formatter_mediator.GetWindowsEventMessage(
          source_name, message_identifier)
      if message_string:
        try:
          event_values[u'message_string'] = message_string.format(*strings)
        except IndexError:
          pass

    message_strings = []
    for string in strings:
      message_strings.append(u'\'{0:s}\''.format(string))
    message_string = u', '.join(message_strings)
    event_values[u'strings'] = u'[{0:s}]'.format(message_string)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(WinEVTXFormatter)



# -*- coding: utf-8 -*-
"""The preprocess plugins manager."""

import logging

from dfvfs.helpers import file_system_searcher
from dfvfs.helpers import windows_path_resolver
from dfwinreg import interface as dfwinreg_interface
from dfwinreg import regf as dfwinreg_regf
from dfwinreg import registry as dfwinreg_registry


class FileSystemWinRegistryFileReader(dfwinreg_interface.WinRegistryFileReader):
  """A file system-based Windows Registry file reader."""

  def __init__(self, file_system, mount_point, environment_variables=None):
    """Initializes a Windows Registry file reader object.

    Args:
      file_system (dfvfs.FileSytem): file system.
      mount_point (dfvfs.PathSpec): mount point path specification.
      environment_variables (Optional[list[EnvironmentVariableArtifact]]):
          environment variables.
    """
    super(FileSystemWinRegistryFileReader, self).__init__()
    self._file_system = file_system
    self._path_resolver = self._CreateWindowsPathResolver(
        file_system, mount_point, environment_variables=environment_variables)

  def _CreateWindowsPathResolver(
      self, file_system, mount_point, environment_variables):
    """Create a Windows path resolver and sets the evironment variables.

    Args:
      file_system (dfvfs.FileSytem): file system.
      mount_point (dfvfs.PathSpec): mount point path specification.
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.

    Returns:
      dfvfs.WindowsPathResolver: Windows path resolver.
    """
    if environment_variables is None:
      environment_variables = []

    path_resolver = windows_path_resolver.WindowsPathResolver(
        file_system, mount_point)

    for environment_variable in environment_variables:
      name = environment_variable.name.lower()
      if name not in (u'systemroot', u'userprofile'):
        continue

      path_resolver.SetEnvironmentVariable(
          environment_variable.name, environment_variable.value)

    return path_resolver

  def _OpenPathSpec(self, path_specification, ascii_codepage=u'cp1252'):
    """Opens the Windows Registry file specified by the path specification.

    Args:
      path_specification (dfvfs.PathSpec): path specfication.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.
    """
    if not path_specification:
      return

    file_entry = self._file_system.GetFileEntryByPathSpec(path_specification)
    if file_entry is None:
      return

    file_object = file_entry.GetFileObject()
    if file_object is None:
      return

    registry_file = dfwinreg_regf.REGFWinRegistryFile(
        ascii_codepage=ascii_codepage)

    try:
      registry_file.Open(file_object)
    except IOError as exception:
      logging.warning(
          u'Unable to open Windows Registry file with error: {0:s}'.format(
              exception))
      file_object.close()
      return

    return registry_file

  def Open(self, path, ascii_codepage=u'cp1252'):
    """Opens the Windows Registry file specified by the path.

    Args:
      path (str): path of the Windows Registry file.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.
    """
    path_specification = self._path_resolver.ResolvePath(path)
    if path_specification is None:
      return

    return self._OpenPathSpec(path_specification)


class PreprocessPluginsManager(object):
  """Class that implements the preprocess plugins manager."""

  _file_system_plugin_classes = {}
  _registry_plugin_classes = {}

  @classmethod
  def _GetPluginObjects(cls, plugin_classes):
    """Returns all plugins.

    Args:
      plugin_classes (dict[str, type]): plugin classes with their class
          name as the key.

    Yields:
      PreprocessPlugin: preprocess plugin.
    """
    for plugin_class in iter(plugin_classes.values()):
      yield plugin_class()

  @classmethod
  def DeregisterPlugin(cls, plugin_class):
    """Deregisters a plugin class.

    Args:
      plugin_class (type): plugin class.

    Raises:
      KeyError: if plugin class is not set for the corresponding name.
    """
    if (plugin_class.__name__ not in cls._file_system_plugin_classes and
        plugin_class.__name__ not in cls._registry_plugin_classes):
      raise KeyError(
          u'Plugin class not set for name: {0:s}.'.format(
              plugin_class.__name__))

    if plugin_class.__name__ in cls._file_system_plugin_classes:
      del cls._file_system_plugin_classes[plugin_class.__name__]

    if plugin_class.__name__ in cls._registry_plugin_classes:
      del cls._registry_plugin_classes[plugin_class.__name__]

  @classmethod
  def RegisterPlugin(cls, plugin_class):
    """Registers a plugin class.

    Args:
      plugin_class (type): plugin class.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
    """
    if (plugin_class.__name__ in cls._file_system_plugin_classes or
        plugin_class.__name__ in cls._registry_plugin_classes):
      raise KeyError(
          u'Plugin class already set for name: {0:s}.'.format(
              plugin_class.__name__))

    if hasattr(plugin_class, u'_REGISTRY_KEY_PATH'):
      cls._registry_plugin_classes[plugin_class.__name__] = plugin_class

    else:
      cls._file_system_plugin_classes[plugin_class.__name__] = plugin_class

  @classmethod
  def RegisterPlugins(cls, plugin_classes):
    """Registers a plugin classes.

    Args:
      plugin_classes (list[type]): plugin classses.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
    """
    for plugin_class in plugin_classes:
      cls.RegisterPlugin(plugin_class)

  @classmethod
  def RunPlugins(cls, file_system, mount_point, knowledge_base):
    """Runs the preprocessing plugins.

    Args:
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      mount_point (dfvfs.PathSpec): mount point path specification that refers
          to the base location of the file system.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
    """
    # TODO: bootstrap the artifact preprocessor.

    searcher = file_system_searcher.FileSystemSearcher(file_system, mount_point)

    for plugin_object in cls._GetPluginObjects(cls._file_system_plugin_classes):
      try:
        plugin_object.Run(searcher, knowledge_base)

      # All exceptions need to be caught here to prevent the manager
      # from being killed by an uncaught exception.
      except Exception as exception:  # pylint: disable=broad-except
        logging.warning(
            u'Preprocess plugin: {0:s} run failed with error: {1:s}'.format(
                plugin_object.plugin_name, exception))

    # Run the Registry plugins separately so we do not have to open
    # Registry files in every plugin.

    environment_variables = None
    if knowledge_base:
      environment_variables = knowledge_base.GetEnvironmentVariables()

    registry_file_reader = FileSystemWinRegistryFileReader(
        file_system, mount_point, environment_variables=environment_variables)
    win_registry = dfwinreg_registry.WinRegistry(
        registry_file_reader=registry_file_reader)

    for plugin_object in cls._GetPluginObjects(cls._registry_plugin_classes):
      try:
        plugin_object.Run(win_registry, knowledge_base)

      # All exceptions need to be caught here to prevent the manager
      # from being killed by an uncaught exception.
      except Exception as exception:  # pylint: disable=broad-except
        logging.warning(
            u'Preprocess plugin: {0:s} run failed with error: {1:s}'.format(
                plugin_object.plugin_name, exception))

    if not knowledge_base.HasUserAccounts():
      logging.warning(u'Unable to find any user accounts on the system.')




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows recycler parsers."""

import unittest

from plaso.formatters import recycler  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import recycler

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class WinRecycleBinParserTest(test_lib.ParserTestCase):
  """Tests for the Windows Recycle Bin parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'$II3DF3L.zip'])
  def testParseVista(self):
    """Tests the Parse function on a Windows Vista RecycleBin file."""
    parser_object = recycler.WinRecycleBinParser()
    storage_writer = self._ParseFile([u'$II3DF3L.zip'], parser_object)

    self.assertEqual(len(storage_writer.events), 1)

    event = storage_writer.events[0]

    expected_filename = (
        u'C:\\Users\\nfury\\Documents\\Alloy Research\\StarFury.zip')
    self.assertEqual(event.original_filename, expected_filename)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-12 20:49:58.633')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.file_size, 724919)

    expected_message = u'{0:s} (from drive: UNKNOWN)'.format(expected_filename)
    expected_message_short = u'Deleted file: {0:s}'.format(expected_filename)
    self._TestGetMessageStrings(event, expected_message, expected_message_short)

  @shared_test_lib.skipUnlessHasTestFile([u'$I103S5F.jpg'])
  def testParseWindows10(self):
    """Tests the Parse function on a Windows 10 RecycleBin file."""
    parser_object = recycler.WinRecycleBinParser()
    storage_writer = self._ParseFile([u'$I103S5F.jpg'], parser_object)

    self.assertEqual(len(storage_writer.events), 1)

    event = storage_writer.events[0]

    expected_filename = (
        u'C:\\Users\\random\\Downloads\\bunnies.jpg')
    self.assertEqual(event.original_filename, expected_filename)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-06-29 21:37:45.618')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.file_size, 222255)

    expected_message = u'{0:s} (from drive: UNKNOWN)'.format(expected_filename)
    expected_message_short = u'Deleted file: {0:s}'.format(expected_filename)
    self._TestGetMessageStrings(event, expected_message, expected_message_short)


class WinRecyclerInfo2ParserTest(test_lib.ParserTestCase):
  """Tests for the Windows Recycler INFO2 parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'INFO2'])
  def testParse(self):
    """Reads an INFO2 file and run a few tests."""
    parser_object = recycler.WinRecyclerInfo2Parser()
    storage_writer = self._ParseFile([u'INFO2'], parser_object)

    self.assertEqual(len(storage_writer.events), 4)

    event = storage_writer.events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2004-08-25 16:18:25.237')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, eventdata.EventTimestamp.DELETED_TIME)

    self.assertEqual(event.record_index, 1)

    expected_filename = (
        u'C:\\Documents and Settings\\Mr. Evil\\Desktop\\lalsetup250.exe')
    self.assertEqual(event.original_filename, expected_filename)

    event = storage_writer.events[1]

    expected_message = (
        u'DC2 -> C:\\Documents and Settings\\Mr. Evil\\Desktop'
        u'\\netstumblerinstaller_0_4_0.exe (from drive: C)')
    expected_message_short = (
        u'Deleted file: C:\\Documents and Settings\\Mr. Evil\\Desktop'
        u'\\netstumblerinstaller...')

    self._TestGetMessageStrings(event, expected_message, expected_message_short)

    event = storage_writer.events[2]

    self._TestGetSourceStrings(event, u'Recycle Bin', u'RECBIN')


if __name__ == '__main__':
  unittest.main()
